@@ -1,5 +1,5 @@
 /*
- * $Id: common.h,v 1.359 2006/01/24 21:17:04 rmanfredi Exp $
+ * $Id: common.h 15162 2008-04-01 22:28:45Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Common header for Gtk-Gnutella.
+ * Common header for gtk-gnutella.
  *
  * @author Richard Eckart
  * @date 2001-2003
@@ -42,12 +42,13 @@
  * Constants
  */
 
+#define GTA_PRODUCT_NAME "gtk-gnutella"	/**< Normally "gtk-gnutella" */
 #define GTA_VERSION 0				/**< major version */
 #define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 0			/**< patch level or teeny version */
-#define GTA_REVISION "stable"			/**< unstable, beta, stable */
-#define GTA_REVCHAR ""			/**< u - unstable, b - beta, none - stable */
-#define GTA_RELEASE "2006-01-24"	/**< ISO 8601 format YYYY-MM-DD */
+#define GTA_PATCHLEVEL 5			/**< patch level or teeny version */
+#define GTA_REVISION "stable"		/**< unstable, beta, stable */
+#define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
+#define GTA_RELEASE "2008-04-01"	/**< ISO 8601 format YYYY-MM-DD */
 #define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
 
 #if defined(USE_GTK1)
@@ -57,11 +58,11 @@
 #elif defined(USE_TOPLESS)
 #define GTA_INTERFACE "Topless"
 #else
-#define GTA_INTERFACE "X11"
+#error "Expected Gtk+ 1.2, Gtk+ 2.x or headless as user-interface."
 #endif
 
 #ifndef HAS_LIBXML2
-#error "You need libxml2 (http://www.xmlsoft.org/) to compile Gtk-Gnutella"
+#error "You need libxml2 (http://www.xmlsoft.org/) to compile gtk-gnutella"
 #endif
 
 /*
@@ -69,22 +70,32 @@
  */
 
 #include <errno.h>
+
+#ifdef I_STDLIB
 #include <stdlib.h>
+#endif
+
 #include <stdio.h>
 #include <signal.h>
+
+#ifdef I_UNISTD
 #include <unistd.h>
+#endif
+
+#ifdef I_SYS_TYPES
 #include <sys/types.h>
+#endif
 
 #ifdef I_SYS_TIME
 #include <sys/time.h>
 #endif
+
 #ifdef I_SYS_TIME_KERNEL
 #define KERNEL
 #include <sys/time.h>
 #undef KERNEL
 #endif
 
-
 #ifdef MINGW32
 #include <ws2tcpip.h>
 #include <winsock.h>
@@ -115,13 +126,23 @@
 #define S_IRWXG _S_IREAD
 #define S_IRWXO _S_IREAD
 
-#else /* !MINGW32 */
+#endif /* !MINGW32 */
 
-#include <sys/resource.h>
+#ifdef I_SYS_SOCKET
 #include <sys/socket.h>
 #endif
 
+#ifdef I_SYS_RESOURCE
+#include <sys/resource.h>
+#endif
+
+#ifdef I_SYS_UN
+#include <sys/un.h>
+#endif
+
+#ifdef I_SYS_STAT
 #include <sys/stat.h>
+#endif
 
 #ifdef MINGW32
 struct iovec 
@@ -146,16 +167,42 @@ struct passwd
 #else /* !MINGW32 */
 
 #include <sys/uio.h>		/* For writev(), readv(), struct iovec */
+#include <sys/wait.h>
+
+#ifdef I_NETINET_IN
 #include <netinet/in.h>
+#endif
+
 #include <netinet/tcp.h>
-#include <arpa/inet.h>		/* For ntohl(), htonl() */
 
 #endif /* MINGW32 */
 
+#ifdef I_ARPA_INET
+#include <arpa/inet.h>		/* For ntohl(), htonl() */
+#endif
+
 #include <ctype.h>
+
+#ifdef I_SYS_FILE
+#include <sys/file.h>
+#endif
+
 #include <fcntl.h>
+
+#if !defined(I_FCNTL) && !defined(I_SYS_FILE)
+#include <sys/fcntl.h>		/* Fallback */
+#endif
+
+#ifdef I_STRING
 #include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#ifdef I_DIRENT
 #include <dirent.h>
+#endif
+
 #include <setjmp.h>
 
 #ifdef I_TIME
@@ -165,18 +212,18 @@ struct passwd
 #ifdef I_SYS_PARAM
 #include <sys/param.h>
 #endif
-#ifdef I_SYS_SYSCTL
-#include <sys/sysctl.h>
-#endif
-#ifdef I_INVENT
-#include <invent.h>
-#endif
 
 #ifdef I_INTTYPES
 #include <inttypes.h>
 #endif /* I_INTTYPES */
 
+#ifdef I_SYS_UTSNAME
+#include <sys/utsname.h>		/* For uname() */
+#endif
+
+#ifdef I_SYS_MMAN
 #include <sys/mman.h>
+#endif
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *) -1)
@@ -192,7 +239,7 @@ struct passwd
  * Proper mmap() support for memory-mapped files requires ISO C functions like
  * sigsetjmp().
  */
-#if defined(__STDC_VERSION__)
+#if defined(HAS_MMAP) && defined(HAS_SIGSETJMP)
 #define USE_MMAP 1
 #endif	/* ISO C */
 
@@ -204,6 +251,18 @@ struct passwd
 #include <netinet/ip.h>
 #endif
 
+#if defined(HAS_POLL)
+#include <poll.h>
+#endif	/* HAS_POLL */
+
+#if !defined(AF_LOCAL) && defined(AF_UNIX)
+#define AF_LOCAL AF_UNIX
+#endif	/* !AF_LOCAL && AF_UNIX */
+
+#if !defined(PF_LOCAL) && defined(PF_UNIX)
+#define PF_LOCAL PF_UNIX
+#endif	/* !PF_LOCAL && PF_UNIX */
+
 /* For pedantic lint checks, define USE_LINT. We override some definitions
  * and hide ``inline'' to prevent certain useless warnings. */
 #ifdef USE_LINT
@@ -232,6 +291,14 @@ struct passwd
 
 #include <glib.h>
 
+#if defined(USE_GLIB1) && !defined(GLIB_MAJOR_VERSION)
+#error "Install GLib 1.2 to compile gtk-gnutella against GLib 1.2."
+#endif
+
+#if defined(USE_GLIB2) && !defined(GLIB_MAJOR_VERSION)
+#error "Install GLib 2.x to compile gtk-gnutella against GLib 2.x."
+#endif
+
 #ifdef USE_LINT
 #undef G_GNUC_INTERNAL
 #define G_GNUC_INTERNAL
@@ -242,10 +309,13 @@ struct passwd
 
 typedef guint64 filesize_t; /**< Use filesize_t to hold filesizes */
 
+#ifdef I_STDARG
 #include <stdarg.h>
-#include <regex.h>
+#endif
 
-#include <zlib.h>
+#ifdef I_REGEX
+#include <regex.h>
+#endif
 
 #ifdef USE_GLIB1
 typedef void (*GCallback) (void);
@@ -255,6 +325,13 @@ typedef void (*GCallback) (void);
 #include <glib-object.h>
 #endif
 
+/*
+ * Array size determination
+ */
+#ifndef G_N_ELEMENTS
+#define G_N_ELEMENTS(arr) (sizeof (arr) / sizeof ((arr)[0]))
+#endif
+
 /*
  * Portability macros.
  */
@@ -288,6 +365,38 @@ typedef void (*GCallback) (void);
 #define STDERR_FILENO 2
 #endif /* STDERR_FILENO */
 
+/*
+ * S_ISLNK() is in POSIX.1-1997 but not in older revisions of the standard.
+ */
+#ifndef S_ISLNK
+#define S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
+#endif	/* S_ISLNK */
+
+/*
+ * These macros determine the maximum/minimum value of the given integer type
+ * "t". This works for signed as well as unsigned types. This code does
+ * carefully avoid integer overflows and undefined behaviour.
+ * However, it's assumed the type consists of exactly sizeof(type) * CHAR_BIT
+ * bits.
+ */
+
+#define MAX_INT_VAL_STEP(t) \
+	((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 1)))
+	 
+#define MAX_INT_VAL(t) \
+	((MAX_INT_VAL_STEP(t) - 1) + MAX_INT_VAL_STEP(t))
+
+#define MIN_INT_VAL(t) \
+	((t) -MAX_INT_VAL(t) - 1)
+
+#ifndef TIME_T_MAX
+/* This assumes time_t is an integer, not a float */
+#define TIME_T_MAX MAX_INT_VAL(time_t)
+#endif /* TIME_T_MAX */
+
+#ifndef OFF_T_MAX
+#define OFF_T_MAX MAX_INT_VAL(off_t)
+#endif /* OFF_T_MAX */
 
 /*
  * Other common macros.
@@ -330,27 +439,16 @@ G_STMT_START {			\
 	}					\
 } G_STMT_END
 
-/**
- * Stores a RCS ID tag inside the object file. Every .c source file should
- * use this macro once as `RCSID("<dollar>Id$")' on top. The ID tag is
- * automagically updated each time the file is committed to the CVS repository.
- * The RCS IDs can be looked up from the compiled binary with e.g. `what',
- * `ident' or `strings'. See also rcs(1) and ident(1).
- */
-#define RCSID(x) \
-static inline const char *	\
-get_rcsid(void)		\
-{	\
-	static const char rcsid[] = "@(#) " x;	\
-	const char *s = rcsid;	\
-	while (*s != '\0') {	\
-		if (*s++ == '$')	\
-			break;	\
-	}	\
-	return s;	\
-}
-
 #if defined(__GNUC__) && defined(__GNUC_MINOR__)
+
+/** HAVE_GCC allows conditionalization depending on the version of gcc
+ *  being used to compile the source.
+ *
+ *  Check each version at "http://gcc.gnu.org/onlinedocs/" for
+ *  support.  Specific functionality may also be broken in some
+ *  compiler revisions, so it is useful to conditionalize on the
+ *  version.
+ */
 #define HAVE_GCC(major, minor) \
 	((__GNUC__ > (major)) || \
 	 (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
@@ -367,7 +465,7 @@ get_rcsid(void)		\
  * void my_memcpy(void *dst, const void *src, size_t n) NON_NULL_PARAM((1, 2));
  */
 #if HAVE_GCC(3, 3)
-#define NON_NULL_PARAM(x) __attribute__((nonnull x))
+#define NON_NULL_PARAM(x) __attribute__((__nonnull__ x))
 #else /* GCC < 3.3 */
 #define NON_NULL_PARAM(x)
 #endif
@@ -377,7 +475,7 @@ get_rcsid(void)		\
  * of GCC do not allow function attributes for function pointers.
  */
 #if HAVE_GCC(3, 0)
-#define PRINTF_FUNC_PTR(x, y) __attribute__((format(printf, (x), (y))))
+#define PRINTF_FUNC_PTR(x, y) __attribute__((format(__printf__, (x), (y))))
 #else /* GCC < 3.0 */
 #define PRINTF_FUNC_PTR(x, y)
 #endif
@@ -385,18 +483,36 @@ get_rcsid(void)		\
 /* Functions using this attribute cause a warning if the returned
  * value is not used. */
 #if HAVE_GCC(3, 4)
-#define WARN_UNUSED_RESULT __attribute__((warn_unused_result))
+#define WARN_UNUSED_RESULT __attribute__((__warn_unused_result__))
 #else /* GCC < 3.4 */
 #define WARN_UNUSED_RESULT
 #endif
 
+/* Instructs the compiler to emit code for this function even if it is
+ * or seems to be unused. */
+#if HAVE_GCC(3, 1)
+#define KEEP_FUNCTION __attribute__((__used__))
+#else /* GCC < 3.1 || !GCC */
+#define KEEP_FUNCTION
+#endif
+
+/* The antidote for WARN_UNUSED_RESULT. This attribute is sometimes
+ * misused for functions that return a result which SHOULD NOT be
+ * ignored in contrast to MUST NOT. Unfortunately, a simple "(void)"
+ * does not suppress this warning.
+ */
+#define IGNORE_RESULT(x) \
+	G_STMT_START { switch (0 != (x)) { default: ; } }  G_STMT_END
+
 /* Functions using this attribute cause a warning if the variable
  * argument list does not contain a NULL pointer. */
+#ifndef G_GNUC_NULL_TERMINATED
 #if HAVE_GCC(4, 0)
-#define WARN_NEED_SENTINEL __attribute__((sentinel))
-#else /* GCC < 4 */
-#define WARN_NEED_SENTINEL
-#endif /* GCC >= 4 */
+#define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
+#else	/* GCC < 4 */
+#define G_GNUC_NULL_TERMINATED
+#endif	/* GCC >= 4 */
+#endif	/* G_GNUC_NULL_TERMINATED */
 
 /* Define G_LIKELY() and G_UNLIKELY() so that they are available when
  * using GLib 1.2 as well. These allow optimization by static branch
@@ -411,11 +527,37 @@ get_rcsid(void)		\
 #endif /* GCC >= 3.4 */
 #endif /* !G_LIKELY */
 
+#ifndef G_GNUC_MALLOC
+#if HAVE_GCC(3, 0)
+#define G_GNUC_MALLOC __attribute__((__malloc__))
+#else
+#define G_GNUC_MALLOC
+#endif	/* GCC >= 3.0 */
+#endif	/* G_GNUC_MALLOC */
+
+#if HAVE_GCC(3, 1)
+#define ALWAYS_INLINE __attribute__((__always_inline__))
+#else
+#define ALWAYS_INLINE
+#endif	/* GCC >= 3.1 */
+
+#ifdef HAS_REGPARM
+#define REGPARM(n)	__attribute__((__regparm__((n))))
+#else
+#define REGPARM(n)
+#endif	/* HAS_REGPARM */
+
 /**
  * CMP() returns the sign of a-b, that means -1, 0, or 1.
  */
 #define CMP(a, b) ((a) == (b) ? 0 : (a) > (b) ? 1 : (-1))
 
+/**
+ * SIGN() returns the sign of an integer value.
+ */
+#define SIGN(x) ((x) == 0 ? 0 : (x) > 0 ? 1 : (-1))
+
+
 /**
  * STATIC_ASSERT() can be used to verify conditions at compile-time. For
  * example, it can be used to ensure that an array has a minimum or exact
@@ -444,15 +586,22 @@ get_rcsid(void)		\
 
 #define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
+#define GTA_BUILD					"$Revision: 15162 $"
 
 /* The next two defines came from huge.h --- Emile */
+#define SHA1_BASE16_SIZE 	40		/**< 160 bits in base16 representation */
 #define SHA1_BASE32_SIZE 	32		/**< 160 bits in base32 representation */
 #define SHA1_RAW_SIZE		20		/**< 160 bits in binary representation */
 
+#define TTH_BASE32_SIZE 	39		/**< 160 bits in base32 representation */
+#define TTH_RAW_SIZE		24		/**< 160 bits in binary representation */
+
 #define BITPRINT_BASE32_SIZE 	72	/**< 352 bits in base32 representation
 									 **  includes dot between sha1 and tiger */
 #define BITPRINT_RAW_SIZE		44	/**< 352 bits in binary representation */
 
+/** Maximum bytes in filename i.e., including NUL */
+#define	FILENAME_MAXBYTES 256
 
 /*
  * Forbidden glib calls.
@@ -510,6 +659,28 @@ ngettext_(const gchar *msg1, const gchar *msg2, gulong n)
  */
 #define NG_(Single, Plural, Number) ngettext_((Single), (Plural), (Number))
 
+/**
+ * Stores a RCS ID tag inside the object file. Every .c source file should
+ * use this macro once as `RCSID("<dollar>Id$")' on top. The ID tag is
+ * automagically updated each time the file is committed to the CVS repository.
+ * The RCS IDs can be looked up from the compiled binary with e.g. `what',
+ * `ident' or `strings'. See also rcs(1) and ident(1).
+ */
+#define RCSID(x) \
+static KEEP_FUNCTION inline const char *	\
+get_rcsid(void)	\
+{	\
+	static const char rcsid[] = "@(#) " x;	\
+	const char *s = rcsid;	\
+	while (*s != '\0') {	\
+		if (*s++ == '$')	\
+			break;	\
+	}	\
+	return s;	\
+}
+
+#include "casts.h"
+
 #endif /* _common_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
