@@ -1,10 +1,8 @@
 /*
- * $Id: mq.h,v 1.5 2005/05/04 17:37:23 graaff Exp $
+ * $Id: mq.h 13015 2007-03-08 19:07:02Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Message queues.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,18 +23,31 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Message queues.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #ifndef _core_mq_h_
 #define _core_mq_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "gnutella.h"
 #include "pmsg.h"
 #include "tx.h"
 
+#include "lib/cq.h"
+#include "lib/slist.h"
+
 typedef struct mqueue mqueue_t;
 
-/*
+/**
  * Operations defined on all mq types.
  */
 
@@ -51,10 +62,11 @@ struct mq_cops {
 	void (*update_flowc)(mqueue_t *q);
 };
 
+enum mq_magic {
+	MQ_MAGIC = 0x33990ee
+};
+
 /**
- * @ingroup core
- * @file
- *
  * A message queue.
  *
  * The queue itself is a two-way list, whose head is kept in `qhead' and the
@@ -73,47 +85,50 @@ struct mq_cops {
  * to be used as a comparison point when speeding up dropping in flow-control.
  */
 struct mqueue {
-	struct gnutella_header header;	/* Comparison point during flow control */
-	struct gnutella_node *node;		/* Node to which this queue belongs */
-	const struct mq_ops *ops;		/* Polymorphic operations */
-	const struct mq_cops *cops;		/* Common operations */
-	txdrv_t *tx_drv;				/* Network TX stack driver */
-	GList *qhead;			/* The queue head, new messages are prepended */
-	GList *qtail;			/* The queue tail, oldest message to send first */
-	GList **qlink;			/* Sorted array of (GList *) entries, or NULL */
-	gpointer swift_ev;		/* Callout queue event in "swift" mode */
-	gint swift_elapsed;		/* Scheduled elapsed time, in ms */
-	gint qlink_count;		/* Amount of entries in `qlink' */
-	gint maxsize;			/* Maximum size of this queue (total queued) */
-	gint count;				/* Amount of messages queued */
-	gint hiwat;				/* High watermark */
-	gint lowat;				/* Low watermark */
-	gint size;				/* Current amount of bytes queued */
-	gint flags;				/* Status flags */
-	gint last_written;		/* Amount last written by service routine */
-	gint flowc_written;		/* Amount written during flow control */
-	gint last_size;			/* Queue size at last "swift" event callback */
+	enum mq_magic magic;	/**< Magic number */
+	gnutella_header_t header;	/**< Comparison point during flow control */
+	struct gnutella_node *node;		/**< Node to which this queue belongs */
+	const struct mq_ops *ops;		/**< Polymorphic operations */
+	const struct mq_cops *cops;		/**< Common operations */
+	txdrv_t *tx_drv;				/**< Network TX stack driver */
+	GList *qhead, *qtail, **qlink;
+	slist_t *qwait;			/**< Waiting queue during putq recursions */
+	cevent_t *swift_ev;		/**< Callout queue event in "swift" mode */
+	gint swift_elapsed;		/**< Scheduled elapsed time, in ms */
+	gint qlink_count;		/**< Amount of entries in `qlink' */
+	gint maxsize;			/**< Maximum size of this queue (total queued) */
+	gint count;				/**< Amount of messages queued */
+	gint hiwat;				/**< High watermark */
+	gint lowat;				/**< Low watermark */
+	gint size;				/**< Current amount of bytes queued */
+	gint flags;				/**< Status flags */
+	gint last_written;		/**< Amount last written by service routine */
+	gint flowc_written;		/**< Amount written during flow control */
+	gint last_size;			/**< Queue size at last "swift" event callback */
+    gint putq_entered;		/**< For recursion checks in mq_putq() */
 };
 
 /*
  * Queue flags.
  */
 
-#define MQ_FLOWC		0x00000001	/* In flow control */
-#define MQ_DISCARD		0x00000002	/* No writing, discard message */
-#define MQ_SWIFT		0x00000004	/* Swift mode, dropping more traffic */
-#define MQ_WARNZONE		0x00000008	/* Between hiwat and lowat */
+enum {
+	MQ_FLOWC	= (1 << 0),	/**< In flow control */
+	MQ_DISCARD	= (1 << 1),	/**< No writing, discard message */
+	MQ_SWIFT	= (1 << 2),	/**< Swift mode, dropping more traffic */
+	MQ_WARNZONE	= (1 << 3)	/**< Between hiwat and lowat */
+};
 
-#define mq_is_flow_controlled(q)	((q)->flags & MQ_FLOWC)
-#define mq_is_swift_controlled(q)	((q)->flags & MQ_SWIFT)
-#define mq_maxsize(q)				((q)->maxsize)
-#define mq_size(q)					((q)->size)
-#define mq_lowat(q)					((q)->lowat)
-#define mq_hiwat(q)					((q)->hiwat)
-#define mq_count(q)					((q)->count)
-#define mq_pending(q)				((q)->size + tx_pending((q)->tx_drv))
-#define mq_bio(q)					(tx_bio_source((q)->tx_drv))
-#define mq_node(q)					((q)->node)
+gboolean mq_is_flow_controlled(const struct mqueue *q);
+gboolean mq_is_swift_controlled(const struct mqueue *q);
+gint mq_maxsize(const struct mqueue *q);
+gint mq_size(const struct mqueue *q);
+gint mq_lowat(const struct mqueue *q);
+gint mq_hiwat(const struct mqueue *q);
+gint mq_count(const struct mqueue *q);
+gint mq_pending(const struct mqueue *q);
+struct bio_source *mq_bio(const struct mqueue *q);
+struct gnutella_node *mq_node(const struct mqueue *q);
 
 /*
  * Public interface
@@ -122,10 +137,25 @@ struct mqueue {
 void mq_putq(mqueue_t *q, pmsg_t *mb);
 void mq_free(mqueue_t *q);
 void mq_clear(mqueue_t *q);
+void mq_discard(mqueue_t *q);
 void mq_shutdown(mqueue_t *q);
 void mq_fill_ops(struct mq_ops *ops);
 
 const struct mq_cops *mq_get_cops(void);
+const gchar *mq_info(const mqueue_t *q);
+
+/*
+ * Message queue assertions.
+ */
+
+#define MQ_DEBUG
+#ifdef MQ_DEBUG
+void mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line);
+
+#define mq_check(x,y)	mq_check_track((x), (y), _WHERE_, __LINE__)
+#else
+#define mq_check(x,y)
+#endif
 
 #endif	/* _core_mq_h_ */
 
