@@ -1,5 +1,5 @@
 /*
- * $Id: routing.h,v 1.3 2005/04/24 18:51:00 cbiere Exp $
+ * $Id: routing.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -26,20 +26,23 @@
 #ifndef _core_routing_h_
 #define _core_routing_h_
 
-#include <glib.h>
+#include "common.h"
 
-/*
+#include "gnutella.h"
+
+/**
  * Route destination types.
  */
 
 typedef enum {
-	ROUTE_NONE = 0,		/* No route, message stops here */
-	ROUTE_ONE,			/* Route to single node */
-	ROUTE_ALL_BUT_ONE,	/* Route to all nodes but one */
-	ROUTE_MULTI			/* Route to list of nodes */
+	ROUTE_NONE = 0,			/**< No route, message stops here */
+	ROUTE_ONE,				/**< Route to single node */
+	ROUTE_ALL_BUT_ONE,		/**< Route to all nodes but one */
+	ROUTE_MULTI,			/**< Route to list of nodes */
+	ROUTE_NO_DUPS_BUT_ONE,	/**< Temporary: watch out broken GTKGs */
 } route_type_t;
 
-/*
+/**
  * Routing destination, as determined by route_message().
  */
 
@@ -47,7 +50,7 @@ struct route_dest {
 	route_type_t type;
 	union {
 		struct gnutella_node *u_node;
-		GSList *u_nodes;				/* For ROUTE_MULTI */
+		GSList *u_nodes;	/**< For ROUTE_MULTI */
 	} ur;
 };
 
@@ -55,20 +58,18 @@ struct route_dest {
  * Global Functions
  */
 
-struct gnutella_header;
-
 void routing_init(void);
 void routing_close(void);
-void message_set_muid(struct gnutella_header *header, guint8 function);
+void message_set_muid(gnutella_header_t *header, guint8 function);
 gboolean route_message(struct gnutella_node **, struct route_dest *);
 void routing_node_remove(struct gnutella_node *);
-void message_add(const gchar *, guint8, struct gnutella_node *);
+void message_add(const gchar *muid, guint8, struct gnutella_node *);
 GSList *route_towards_guid(const gchar *guid);
-gboolean route_exists_for_reply(gchar *muid, guint8 function);
+gboolean route_exists_for_reply(const gchar *muid, guint8 function);
 
-gboolean route_proxy_add(gchar *guid, struct gnutella_node *n);
-void route_proxy_remove(gchar *guid);
-struct gnutella_node *route_proxy_find(gchar *guid);
+gboolean route_proxy_add(const gchar *guid, struct gnutella_node *n);
+void route_proxy_remove(const gchar *guid);
+struct gnutella_node *route_proxy_find(const gchar *guid);
 
 #endif /* _core_routing_h_ */
 /* vi: set ts=4 sw=4 cindent: */
