@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.h 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: sockets.h 15053 2007-12-08 01:15:45Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -108,9 +108,7 @@ struct gnutella_socket {
 	enum socket_direction direction;
 	enum socket_type type;
 	enum net_type net;
-	gboolean omit_token;	/**< TRUE if the connection needs no token */
-	gboolean corked;
-	gboolean was_shutdown;	/**< Set if shutdown() was used */
+
 	gint adns;				/**< status of ADNS resolution */
 	const gchar *adns_msg;	/**< ADNS error message */
 
@@ -123,9 +121,7 @@ struct gnutella_socket {
 
 	struct wrap_io wio;		/**< Wrapped IO object */
 
-#ifdef HAS_GNUTLS
 	struct socket_tls_ctx tls;
-#endif /* HAS_GNUTLS */
 
 	union {
 		struct gnutella_node *node;
@@ -166,35 +162,21 @@ extern struct gnutella_socket *s_local_listen;
  */
 
 static inline gboolean
-socket_with_tls(struct gnutella_socket *s)
-#ifdef HAS_GNUTLS
+socket_with_tls(const struct gnutella_socket *s)
 {
 	return s->tls.enabled && s->tls.stage >= SOCK_TLS_INITIALIZED;
 }
-#else
-{
-	(void) s;
-	return FALSE;
-}
-#endif	/* HAS_GNUTLS */
 
 static inline gboolean
-socket_uses_tls(struct gnutella_socket *s)
-#ifdef HAS_GNUTLS
+socket_uses_tls(const struct gnutella_socket *s)
 {
-	return s->tls.enabled && s->tls.stage >= SOCK_TLS_ESTABLISHED;
+	return s->tls.enabled && s->tls.stage == SOCK_TLS_ESTABLISHED;
 }
-#else
-{
-	(void) s;
-	return FALSE;
-}
-#endif	/* HAS_GNUTLS */
 
 static inline gboolean
-sock_is_corked(const struct gnutella_socket *s)
+socket_is_corked(const struct gnutella_socket *s)
 {
-	return s->corked;
+	return 0 != (SOCK_F_CORKED & s->flags);
 }
 
 /**
@@ -237,15 +219,16 @@ void socket_evt_set(struct gnutella_socket *s,
 	inputevt_cond_t cond, inputevt_handler_t handler, gpointer data);
 void socket_evt_clear(struct gnutella_socket *s);
 
-void sock_cork(struct gnutella_socket *s, gboolean on);
-void sock_send_buf(struct gnutella_socket *s, gint size, gboolean shrink);
-void sock_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink);
-void sock_nodelay(struct gnutella_socket *s, gboolean on);
-void sock_tx_shutdown(struct gnutella_socket *s);
+void socket_cork(struct gnutella_socket *s, gboolean on);
+void socket_send_buf(struct gnutella_socket *s, gint size, gboolean shrink);
+void socket_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink);
+void socket_nodelay(struct gnutella_socket *s, gboolean on);
+void socket_tx_shutdown(struct gnutella_socket *s);
 void socket_tos_default(const struct gnutella_socket *s);
 void socket_tos_throughput(const struct gnutella_socket *s);
 void socket_tos_lowdelay(const struct gnutella_socket *s);
 void socket_tos_normal(const struct gnutella_socket *s);
+void socket_set_quickack(struct gnutella_socket *s, int val);
 gboolean socket_bad_hostname(struct gnutella_socket *s);
 void socket_disable_token(struct gnutella_socket *s);
 gboolean socket_omit_token(struct gnutella_socket *s);
