@@ -1,5 +1,5 @@
 /*
- * $Id: spam.c 12062 2006-09-27 20:09:08Z cbiere $
+ * $Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  * Copyright (c) 2004, Raphael Manfredi
@@ -41,13 +41,13 @@
 
 #include "common.h"
 
-RCSID("$Id: spam.c 12062 2006-09-27 20:09:08Z cbiere $")
+RCSID("$Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $")
 
-#include "gdb.h"
 #include "spam.h"
 #include "settings.h"
 #include "nodes.h"
 
+#include "lib/halloc.h"
 #include "lib/atoms.h"
 #include "lib/bit_array.h"
 #include "lib/file.h"
@@ -56,6 +56,7 @@ RCSID("$Id: spam.c 12062 2006-09-27 20:09:08Z cbiere $")
 #include "lib/glib-missing.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
+#include "lib/utf8.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
@@ -66,40 +67,91 @@ RCSID("$Id: spam.c 12062 2006-09-27 20:09:08Z cbiere $")
 static const gchar spam_text_file[] = "spam.txt";
 static const gchar spam_what[] = "Spam database";
 
-struct spam_lut {
+/****** BEGIN IDEAS ONLY ******/
+
+/*
+ * The following hash tables function in pairs, which are "rotated" on a
+ * regular basis, the currently active table becoming the old one and the
+ * old one being cleared.  This provides a way to store temporary information
+ * that will naturally fade out if unused.
+ */
+
+/*
+ * These tables keep track of the association between a GUID and the
+ * last known IP:port of the servent.  If more than one association is found,
+ * then we most probably face a spamming entity using the same GUID accross
+ * a range of different IP addresses..
+ */
+
+/*
+ * These tables keep track of the association between an IP:port and a servent
+ * GUID.  If more than one association is found, then we most probably face a
+ * spamming entity using random GUIDs.
+ */
+
+/*
+ * These tables keep track of the amount of query hits we have seen from a given
+ * GUID and IP:port for a given query MUID, and the total amount of entries
+ * that these hits contained.
+ */
+
+/*
+ * These tables keep track of all the different ports we see behind a given
+ * IP address.  Too many ports are likely to indicate a spamming origin.
+ */
+
+/*
+ * These tables keep track of all the various vendor codes we see originating
+ * from a given IP:port.
+ */
+
+/*
+ * These tables keep track of the amount of query hits originating from a /24
+ * network for a given MUID for one pair, and the various /24 networks that
+ * have replied to a given MUID in another pair.
+ */
+
+/****** END IDEAS ONLY ******/
+
+/* NOTE: This is disabled because SQLite does not seem worth the overhead
+ *       here for now as spam.txt isn't horribly large. With a slow disk
+ *		 or NFS, for example, the performance hit may be noticable.
+ */
+#if 0
 #ifdef HAS_SQLITE
-	struct gdb_stmt *lookup_stmt;
-	struct gdb_stmt *insert_stmt;
-#else  /* HAS_SQLITE */
-	GHashTable *ht;
+#define USE_SQLITE 1
 #endif /* HAS_SQLITE */
+#endif
+
+struct spam_lut {
+	GSList *sl_names;	/* List of g_malloc()ed regex_t items */
 };
 
-struct spam_lut spam_lut;
+static struct spam_lut spam_lut;
 
 typedef enum {
 	SPAM_TAG_UNKNOWN = 0,
 	SPAM_TAG_ADDED,
 	SPAM_TAG_END,
+	SPAM_TAG_NAME,
 	SPAM_TAG_SHA1,
+	SPAM_TAG_SIZE,
 
 	NUM_SPAM_TAGS
 } spam_tag_t;
 
-struct spam_item {
-	gchar sha1[SHA1_RAW_SIZE];
-};
-
 static const struct spam_tag {
 	spam_tag_t	tag;
 	const gchar *str;
 } spam_tag_map[] = {
 	/* Must be sorted alphabetically for dichotomic search */
 
-#define SPAM_TAG(x) { CAT2(SPAM_TAG_,x), STRINGIFY(x) }
+#define SPAM_TAG(x) { CAT2(SPAM_TAG_,x), #x }
 	SPAM_TAG(ADDED),
 	SPAM_TAG(END),
+	SPAM_TAG(NAME),
 	SPAM_TAG(SHA1),
+	SPAM_TAG(SIZE),
 
 	/* Above line intentionally left blank (for "!}sort" on vi) */
 #undef SPAM_TAG
@@ -109,7 +161,7 @@ static const struct spam_tag {
 static spam_tag_t
 spam_string_to_tag(const gchar *s)
 {
-	STATIC_ASSERT(G_N_ELEMENTS(spam_tag_map) == (NUM_SPAM_TAGS - 1));
+	STATIC_ASSERT(G_N_ELEMENTS(spam_tag_map) == NUM_SPAM_TAGS - 1U);
 
 #define GET_ITEM(i) (spam_tag_map[(i)].str)
 #define FOUND(i) G_STMT_START { \
@@ -126,106 +178,48 @@ spam_string_to_tag(const gchar *s)
 	return SPAM_TAG_UNKNOWN;
 }
 
-static gint
-spam_db_open(struct spam_lut *lut)
-#ifdef HAS_SQLITE
+struct namesize_item {
+	regex_t		pattern;
+	filesize_t	min_size;
+	filesize_t	max_size;
+};
+
+static gboolean 
+spam_add_name_and_size(const gchar *name,
+	filesize_t min_size, filesize_t max_size)
 {
-	g_assert(lut);
+	struct namesize_item *item;
+	int error;
 
-	{
-		static const gchar cmd[] =
-			"CREATE TABLE IF NOT EXISTS spam (sha1 BLOB PRIMARY KEY);"
-			"DELETE FROM spam;";
-		gchar *errmsg;
-		gint ret;
+	g_return_val_if_fail(name, TRUE);
+	g_return_val_if_fail(min_size <= max_size, TRUE);
 
-		ret = gdb_exec(cmd, &errmsg);
-		if (0 != ret) {
-			g_warning("gdb_exec() failed: %s", errmsg);
-			gdb_free(errmsg);
-			goto failure;
-		}
-		ret = gdb_declare_types("spam", "sha1", GDB_CT_SHA1, (void *) 0);
-		if (0 != ret) {
-			g_warning("gdb_declare_types() failed: %s", gdb_error_message());
-			goto failure;
-		}
+	item = walloc(sizeof *item);
+	error = regcomp(&item->pattern, name, REG_EXTENDED | REG_NOSUB);
+	if (error) {
+		gchar buf[1024];
+
+		regerror(error, &item->pattern, buf, sizeof buf);
+		g_warning("spam_add_name_and_size(): regcomp() failed: %s", buf);
+		regfree(&item->pattern);
+		wfree(item, sizeof *item);
+		return TRUE;
+	} else {
+		item->min_size = min_size;
+		item->max_size = max_size;
+		spam_lut.sl_names = g_slist_prepend(spam_lut.sl_names, item);
+		return FALSE;
 	}
-
-	{
-		static const gchar cmd[] =
-			"INSERT INTO spam VALUES($x);";
-	  
-		if (0 != gdb_stmt_prepare(cmd, &lut->insert_stmt)) {
-			g_warning("gdb_prepare() failed: %s",
-				gdb_error_message());
-			goto failure;
-		}
-	}
-
-	{
-		static const gchar cmd[] =
-			"SELECT OID FROM spam WHERE sha1 = $x;";
-
-		if (0 != gdb_stmt_prepare(cmd, &lut->lookup_stmt)) {
-			g_warning("gdb_prepare() failed: %s",
-				gdb_error_message());
-			goto failure;
-		}
-	}
-	
-	return 0;
-	
-failure:
-	spam_close();
-	return -1;
-}
-#else	/* !HAS_SQLITE */
-{
-	g_assert(lut);
-
-	lut->ht = g_hash_table_new(sha1_hash, sha1_eq);
-	return 0;
-}
-#endif	/* HAS_SQLITE */
-
-void
-spam_add(const struct spam_item *item)
-{
-	g_assert(item);
-
-#ifdef HAS_SQLITE
-	if (spam_lut.insert_stmt) {
-		struct gdb_stmt *stmt = spam_lut.insert_stmt;
-		gint ret;
-		
-		ret = gdb_stmt_reset(stmt);
-		if (0 == ret) {
-			ret = gdb_stmt_bind_static_blob(stmt,
-					1, item->sha1, sizeof item->sha1);
-			if (0 == ret) {
-				ret = gdb_stmt_step(stmt);
-				if (GDB_STEP_DONE != ret) {
-					g_warning("%s: gdb_stmt_step() failed: %s",
-						"spam_add", gdb_error_message());
-				}
-			} else {
-				g_warning("%s: gdb_stmt_bind_static_blob() failed: %s",
-					"spam_add", gdb_error_message());
-			}
-		} else {
-			g_warning("%s: gdb_stmt_reset() failed: %s",
-				"spam_add", gdb_error_message());
-		}
-	}
-#else	/* HAS_SQLITE */
-	if (spam_lut.ht) {
-		gchar *sha1 = atom_sha1_get(item->sha1);
-		g_hash_table_insert(spam_lut.ht, sha1, sha1);
-	}
-#endif	/* HAS_SQLITE */
 }
 
+struct spam_item {
+	struct sha1 sha1;
+	gchar		*name;
+	filesize_t  min_size;
+	filesize_t  max_size;
+	gboolean	done;
+	gboolean	damaged;
+};
 
 /**
  * Load spam database from the supplied FILE.
@@ -239,7 +233,7 @@ spam_add(const struct spam_item *item)
  *
  * @returns the amount of entries loaded or -1 on failure.
  */
-static glong
+static gulong
 spam_load(FILE *f)
 {
 	static const struct spam_item zero_item;
@@ -247,30 +241,21 @@ spam_load(FILE *f)
 	gchar line[1024];
 	guint line_no = 0;
 	bit_array_t tag_used[BIT_ARRAY_SIZE(NUM_SPAM_TAGS)];
-	gboolean done = FALSE;
 	gulong item_count = 0;
 
 	g_assert(f);
 
 	/* Reset state */
-	done = FALSE;
 	item = zero_item;
-	bit_array_clear_range(tag_used, 0, (guint) NUM_SPAM_TAGS - 1);
-
-	if (0 != spam_db_open(&spam_lut)) {
-		return -1;
-	}
-	gdb_begin();
+	bit_array_clear_range(tag_used, 0, NUM_SPAM_TAGS - 1U);
 
 	while (fgets(line, sizeof line, f)) {
 		const gchar *tag_name, *value;
 		gchar *sp, *nl;
-		gboolean damaged;
 		spam_tag_t tag;
 
 		line_no++;
 
-		damaged = FALSE;
 		nl = strchr(line, '\n');
 		if (!nl) {
 			/*
@@ -300,11 +285,12 @@ spam_load(FILE *f)
 		tag_name = line;
 
 		tag = spam_string_to_tag(tag_name);
-		g_assert((gint) tag >= 0 && tag < NUM_SPAM_TAGS);
+		g_assert(UNSIGNED(tag) < UNSIGNED(NUM_SPAM_TAGS));
+
 		if (SPAM_TAG_UNKNOWN != tag && !bit_array_flip(tag_used, tag)) {
 			g_warning("spam_load(): duplicate tag \"%s\" in entry in line %u",
 				tag_name, line_no);
-			break;
+			continue;
 		}
 		
 		switch (tag) {
@@ -314,7 +300,7 @@ spam_load(FILE *f)
 				
 				t = date2time(value, tm_time());
 				if ((time_t) -1 == t) {
-					damaged |= TRUE;
+					item.damaged = TRUE;
 				}
 			}
 			break;
@@ -322,30 +308,80 @@ spam_load(FILE *f)
 		case SPAM_TAG_SHA1:
 			{
 				if (strlen(value) != SHA1_BASE32_SIZE) {
-					damaged = TRUE;
+					item.damaged = TRUE;
 					g_warning("spam_load(): SHA-1 has wrong length.");
 				} else {
-					const gchar *raw;
+					const struct sha1 *raw;
 
 					raw = base32_sha1(value);
-					if (!raw)
-						damaged = TRUE;
+					if (raw)
+						item.sha1 = *raw;
 					else
-						memcpy(item.sha1, raw, sizeof item.sha1);
+						item.damaged = TRUE;
+				}
+			}
+			break;
+
+		case SPAM_TAG_NAME:
+			{
+				if ('\0' == value[0]) {
+					item.damaged = TRUE;
+					g_warning("spam_load(): Missing filename pattern.");
+				} else if (!utf8_is_valid_string(value)) {
+					item.damaged = TRUE;
+					g_warning("spam_load(): Filename pattern is not UTF-8.");
+				} else {
+					item.name = g_strdup(value);
+				}
+			}
+			break;
+
+		case SPAM_TAG_SIZE:
+			{
+				const gchar *endptr;
+				guint64 u;
+				gint error;
+					
+				u = parse_uint64(value, &endptr, 10, &error);
+				if (error) {
+					item.damaged = TRUE;
+					g_warning("spam_load(): Cannot parse SIZE: %s", value);
+				} else {
+					item.min_size = u;
+					item.max_size = u;
+
+					if ('-' == endptr[0]) {
+						u = parse_uint64(&endptr[1], &endptr, 10, &error);
+						if (error) {
+							item.damaged = TRUE;
+							g_warning("spam_load(): Cannot parse SIZE: %s",
+								value);
+						}
+						if (u < item.min_size) {
+							item.damaged = TRUE;
+							g_warning("spam_load(): "
+								"Maximum size below minimum size");
+						} else {
+							item.max_size = u;
+						}
+					}
 				}
 			}
 			break;
 
 		case SPAM_TAG_END:
-			if (!bit_array_get(tag_used, SPAM_TAG_SHA1)) {
-				g_warning("spam_load(): missing SHA1 tag");
-				damaged = TRUE;
+			if (
+				!bit_array_get(tag_used, SPAM_TAG_SHA1) &&
+				!bit_array_get(tag_used, SPAM_TAG_NAME)
+			) {
+				g_warning("spam_load(): missing SHA1 or NAME tag");
+				item.damaged = TRUE;
 			}
 			if (!bit_array_get(tag_used, SPAM_TAG_ADDED)) {
 				g_warning("spam_load(): missing ADDED tag");
-				damaged = TRUE;
+				item.damaged = TRUE;
 			}
-			done = TRUE;
+			item.done = TRUE;
 			break;
 
 		case SPAM_TAG_UNKNOWN:
@@ -357,31 +393,42 @@ spam_load(FILE *f)
 			break;
 		}
 
-		if (damaged) {
+		if (item.done && !item.damaged) {
+			if (bit_array_get(tag_used, SPAM_TAG_SHA1)) {
+				spam_sha1_add(&item.sha1);
+				item_count++;
+			}
+			if (bit_array_get(tag_used, SPAM_TAG_NAME)) {
+				if (!bit_array_get(tag_used, SPAM_TAG_SIZE)) {
+					item.min_size = 0;
+					item.max_size = MAX_INT_VAL(filesize_t);
+				}
+				if (
+					spam_add_name_and_size(item.name,
+						item.min_size, item.max_size)
+				) {
+					item.damaged = TRUE;	
+				} else {
+					item_count++;
+				}
+			}
+		}
+
+		if (item.damaged) {
 			g_warning("Damaged spam entry in line %u: "
 				"tag_name=\"%s\", value=\"%s\"",
 				line_no, tag_name, value);
-			break;
 		}
 
-		if (done) {
-			if (spam_check(item.sha1)) {
-				g_warning(
-					"Ignoring duplicate spam item around line %u (sha1=%s)",
-				   	line_no, sha1_base32(item.sha1));
-			} else {
-				spam_add(&item);
-				item_count++;
-			}
-			
+		if (item.done) {
 			/* Reset state */
-			done = FALSE;
+			G_FREE_NULL(item.name);
 			item = zero_item;
-			bit_array_clear_range(tag_used, 0, (guint) NUM_SPAM_TAGS - 1);
+			bit_array_clear_range(tag_used, 0, NUM_SPAM_TAGS - 1U);
 		}
 	}
 
-	gdb_commit();
+	spam_sha1_sync();
 
 	return item_count;
 }
@@ -435,14 +482,20 @@ spam_retrieve_from_file(FILE *f, const gchar *path, const gchar *filename)
 static void
 spam_retrieve(void)
 {
+	static file_path_t fp[3];
+	guint num_fp = G_N_ELEMENTS(fp) - 1;
 	FILE *f;
 	gint idx;
-	file_path_t fp[1];
-
+	
 	file_path_set(&fp[0], settings_config_dir(), spam_text_file);
-	f = file_config_open_read_norename_chosen(spam_what,
-			fp, G_N_ELEMENTS(fp), &idx);
+	file_path_set(&fp[1], PRIVLIB_EXP, spam_text_file);
 
+#ifndef OFFICIAL_BUILD
+	file_path_set(&fp[2], PACKAGE_EXTRA_SOURCE_DIR, spam_text_file);
+	num_fp++;
+#endif	/* !OFFICIAL_BUILD */
+
+	f = file_config_open_read_norename_chosen(spam_what, fp, num_fp, &idx);
 	if (f) {
 		spam_retrieve_from_file(f, fp[idx].dir, fp[idx].name);
 		fclose(f);
@@ -455,84 +508,55 @@ spam_retrieve(void)
 void
 spam_init(void)
 {
+	spam_sha1_init();
 	spam_retrieve();
 }
 
-#ifndef HAS_SQLITE
-static void
-spam_item_free(gpointer key, gpointer value, gpointer unused_x)
-{
-	(void) unused_x;
-
-	g_assert(key == value);
-	atom_sha1_free(key);
-}
-#endif /* HAS_SQLITE */
-
 /**
  * Frees all entries in the spam database.
  */
 void
 spam_close(void)
 {
-#ifdef HAS_SQLITE
-	gdb_stmt_finalize(&spam_lut.insert_stmt);
-	gdb_stmt_finalize(&spam_lut.lookup_stmt);
-#else /* HAS_SQLITE */
-	if (spam_lut.ht) {
-		g_hash_table_foreach(spam_lut.ht, spam_item_free, NULL);
-		g_hash_table_destroy(spam_lut.ht);
-		spam_lut.ht = NULL;
+	GSList *sl;
+
+	for (sl = spam_lut.sl_names; NULL != sl; sl = g_slist_next(sl)) {
+		struct namesize_item *item = sl->data;
+
+		g_assert(item);
+		regfree(&item->pattern);
+		wfree(item, sizeof *item);
 	}
-#endif /* HAS_SQLITE */
+	g_slist_free(spam_lut.sl_names);
+	spam_lut.sl_names = NULL;
+	spam_sha1_close();
 }
 
 /**
- * Check the given SHA-1 against the spam database.
+ * Check the given filename against the spam database.
  *
- * @param sha1 the SHA-1 to check.
+ * @param filename the filename to check.
  * @returns TRUE if found, and FALSE if not.
  */
 gboolean
-spam_check(const char *sha1)
+spam_check_filename_and_size(const char *filename, filesize_t size)
 {
-	g_assert(sha1);
+	const GSList *sl;
 
-#ifdef HAS_SQLITE
-	if (spam_lut.lookup_stmt) {
-		struct gdb_stmt *stmt;
-		gint ret;
-	
-		stmt = spam_lut.lookup_stmt;	
-		ret = gdb_stmt_reset(stmt);
-		if (0 == ret) {
-			ret = gdb_stmt_bind_static_blob(stmt, 1, sha1, SHA1_RAW_SIZE);
-			if (0 == ret) {
-				enum gdb_step step;
-				
-				step = gdb_stmt_step(stmt);
-				if (GDB_STEP_ROW == step) {
-					return TRUE;
-				}
-				if (GDB_STEP_DONE != step) {
-					g_warning("%s: gdb_step() failed: %s",
-						"spam_check", gdb_error_message());
-				}
-			} else {
-				g_warning("%s: gdb_stmt_bind_static_blob() failed: %s",
-					"spam_check", gdb_error_message());
-			}
-		} else {
-			g_warning("%s: gdb_stmt_reset() failed: %s",
-				"spam_check", gdb_error_message());
+	g_return_val_if_fail(filename, FALSE);
+
+	for (sl = spam_lut.sl_names; NULL != sl; sl = g_slist_next(sl)) {
+		const struct namesize_item *item = sl->data;
+
+		g_assert(item);
+		if (
+			size >= item->min_size &&
+			size <= item->max_size &&
+			0 == regexec(&item->pattern, filename, 0, NULL, 0)
+		) {
+			return TRUE;
 		}
-		return FALSE;
 	}
-#else	/* HAS_SQLITE */
-	if (spam_lut.ht) {
-		return NULL != g_hash_table_lookup(spam_lut.ht, sha1);
-	}
-#endif	/* HAS_SQLITE */
 	return FALSE;
 }
 
