@@ -1,5 +1,5 @@
 /*
- * $Id: thex_download.c 13895 2007-06-20 19:32:53Z cbiere $
+ * $Id: thex_download.c 14877 2007-09-10 11:50:11Z cbiere $
  *
  * Copyright (c) 2005, Raphael Manfredi
  * Copyright (c) 2005, Martijn van Oosterhout <kleptog@svana.org>
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: thex_download.c 13895 2007-06-20 19:32:53Z cbiere $")
+RCSID("$Id: thex_download.c 14877 2007-09-10 11:50:11Z cbiere $")
 
 #include <libxml/parser.h>                                                      
 #include <libxml/tree.h>                                                        
@@ -50,6 +50,8 @@ RCSID("$Id: thex_download.c 13895 2007-06-20 19:32:53Z cbiere $")
 #include "thex.h"
 #include "thex_download.h"
 
+#include "if/gnet_property_priv.h"
+
 #include "lib/atoms.h"
 #include "lib/endian.h"
 #include "lib/tigertree.h"
@@ -215,14 +217,18 @@ verify_element(xmlNode *node, const char *prop, const char *expect)
 	
 	value = STRTRACK(xml_get_string(node, prop));
   	if (NULL == value) {
-    	g_message("Couldn't find property \"%s\" of node \"%s\"",
-			prop, node->name);
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Couldn't find property \"%s\" of node \"%s\"",
+				prop, node->name);
+		}
 		goto finish;
 	}
 	if (0 != strcmp(value, expect)) {
-		g_message("Property %s/%s doesn't match expected value \"%s\", "
-			"got \"%s\"",
-			node->name, prop, expect, value);
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Property %s/%s doesn't match expected value \"%s\", "
+				"got \"%s\"",
+				node->name, prop, expect, value);
+		}
 		goto finish;
 	}
 	success = TRUE;
@@ -242,20 +248,26 @@ thex_download_handle_xml(struct thex_download *ctx,
 	gboolean success = FALSE;
 
 	if (size <= 0) {
-		g_message("XML record has no data");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("XML record has no data");
+		}
 		goto finish;
 	}
 	
 	doc = xmlReadMemory(data, size, "noname.xml", NULL, 0);
 	if (NULL == doc) {
-		g_message("Cannot parse XML record");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Cannot parse XML record");
+		}
 		goto finish;
 	}
 	root = xmlDocGetRootElement(doc);
 	
   	hashtree = find_element_by_name(root, "hashtree");
 	if (NULL == hashtree) {
-		g_message("Couldn't find hashtree element");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Couldn't find hashtree element");
+		}
 		goto finish;
 	}
 	
@@ -266,7 +278,9 @@ thex_download_handle_xml(struct thex_download *ctx,
 		if (!verify_element(node, "segmentsize", THEX_SEGMENT_SIZE))
 			goto finish;
 	} else {
-		g_message("Couldn't find hashtree/file element");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Couldn't find hashtree/file element");
+		}
 		goto finish;
 	}
 
@@ -277,13 +291,14 @@ thex_download_handle_xml(struct thex_download *ctx,
 		if (!verify_element(node, "outputsize", THEX_HASH_SIZE))
 			goto finish;
 	} else {
-		g_message("Couldn't find hashtree/digest element");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Couldn't find hashtree/digest element");
+		}
     	goto finish;
 	}
   
 	node = find_element_by_name(hashtree->children, "serializedtree");
 	if (node) {
-		guint32 depth;
 		char *value;
 		int error;
 		
@@ -292,8 +307,10 @@ thex_download_handle_xml(struct thex_download *ctx,
 
 		value = STRTRACK(xml_get_string(node, "uri"));
 		if (NULL == value) {
-			g_message("Couldn't find property \"uri\" of node \"%s\"",
-				node->name);
+			if (GNET_PROPERTY(tigertree_debug)) {
+				g_message("Couldn't find property \"uri\" of node \"%s\"",
+					node->name);
+			}
 			goto finish;
 		}
 		hashtree_id = g_strdup(value);
@@ -301,28 +318,26 @@ thex_download_handle_xml(struct thex_download *ctx,
 
 		value = STRTRACK(xml_get_string(node, "depth"));
 		if (NULL == value) {
-			g_message("Couldn't find property \"depth\" of node \"%s\"",
-				node->name);
+			if (GNET_PROPERTY(tigertree_debug)) {
+				g_message("Couldn't find property \"depth\" of node \"%s\"",
+					node->name);
+			}
 			goto finish;
 		}
 		
-		depth = parse_uint32(value, NULL, 10, &error);
-		error |= depth > tt_full_depth(ctx->filesize);
+		ctx->depth = parse_uint16(value, NULL, 10, &error);
+		error |= ctx->depth > tt_full_depth(ctx->filesize);
 		if (error) {
+			ctx->depth = 0;
 			g_message("Bad value for \"depth\" of node \"%s\": \"%s\"",
 				node->name, value);
 		}
 		xml_string_free(&value);
 		if (error)
 			goto finish;
-
-		if (depth < tt_good_depth(ctx->filesize)) {
-			g_message("Tree depth is below the \"good\" depth");
-			goto finish;
-		}
-		ctx->depth = MIN(depth, TTH_MAX_DEPTH);
 	} else {
-		g_message("Couldn't find hashtree/serializedtree element");
+		if (GNET_PROPERTY(tigertree_debug))
+			g_message("Couldn't find hashtree/serializedtree element");
 		goto finish;
 	}
 
@@ -345,29 +360,65 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 {
 	gboolean success = FALSE;
 	size_t n_nodes, n_leaves, n, start;
+	unsigned good_depth, min_depth;
 	const struct tth *leaves;
 	struct tth tth;
 
 	if (size <= 0) {
-		g_message("Hashtree record has no data");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree record has no data");
+		}
 		goto finish;
 	}
 	if (size < TTH_RAW_SIZE) {
-		g_message("Hashtree record is too small");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree record is too small");
+		}
 		goto finish;
 	}
 	if (size % TTH_RAW_SIZE) {
-		g_message("Hashtree has bad size");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree has bad size");
+		}
 		goto finish;
 	}
 	memcpy(tth.data, data, TTH_RAW_SIZE);
 	if (!tth_eq(&tth, ctx->tth)) {
-		g_message("Hashtree has different root hash %s", tth_base32(&tth));
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree has different root hash %s", tth_base32(&tth));
+		}
 		goto finish;
 	}
 
-	n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
 	n_nodes = size / TTH_RAW_SIZE;
+	n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
+
+	/* Shareaza use a fixed depth of 9, allow one level less like others */
+	good_depth = tt_good_depth(ctx->filesize);
+	min_depth = good_depth - (good_depth > 0);
+
+	ctx->depth = MIN(ctx->depth, good_depth);
+	if (n_nodes < n_leaves * 2 - 1) {
+		ctx->depth = good_depth;
+		n = tt_node_count_at_depth(ctx->filesize, ctx->depth);
+		n = n * 2 - 1; /* All nodes, not just leaves */
+		while (n > n_nodes) {
+			n = (n + 1) / 2;
+			ctx->depth--;
+		}
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Calculated depth of hashtree: %u", ctx->depth);
+		}
+		n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
+	}
+
+	if (ctx->depth < min_depth) {
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Tree depth (%u) is below the acceptable depth (%u)",
+				ctx->depth, min_depth);
+		}
+		goto finish;
+	}
 
 	start = 0;
 	n = n_leaves;
@@ -377,8 +428,14 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 	}
 
 	if (n_nodes < start + n_leaves) {
-		g_message("Hashtree has too few nodes (nodes=%u, depth=%u)",
-			(unsigned) n_nodes, ctx->depth);
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree has too few nodes "
+				"(filesize=%s depth=%u nodes=%lu expected=%lu)",
+				filesize_to_string(ctx->filesize),
+				ctx->depth,
+				(unsigned long) n_nodes,
+				(unsigned long) n_leaves * 2 - 1);
+		}
 		goto finish;
 	}
 	
@@ -387,7 +444,9 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 
 	tth = tt_root_hash(leaves, n_leaves);
 	if (!tth_eq(&tth, ctx->tth)) {
-		g_message("Hashtree does not match root hash %s", tth_base32(&tth));
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Hashtree does not match root hash %s", tth_base32(&tth));
+		}
 		goto finish;
 	}
 
@@ -399,6 +458,23 @@ finish:
 	return success;
 }
 
+static void
+thex_dump_dime_records(const GSList *records)
+{
+	const GSList *iter;
+
+	for (iter = records; NULL != iter; iter = g_slist_next(iter)) {
+		const struct dime_record *record;
+
+		record = iter->data;
+		g_assert(record);
+		dump_hex(stderr, "THEX DIME record type",
+			dime_record_type(record), dime_record_type_length(record));
+		dump_hex(stderr, "THEX DIME record ID",
+			dime_record_id(record), dime_record_id_length(record));
+	}
+}
+
 static const struct dime_record *
 dime_find_record(const GSList *records, const char *type, const char *id)
 {
@@ -417,7 +493,7 @@ dime_find_record(const GSList *records, const char *type, const char *id)
 		
 		record = iter->data;
 		g_assert(record);
-
+		
 		if (dime_record_type_length(record) != type_length)
 			continue;
 		if (0 != ascii_strncasecmp(dime_record_type(record), type, type_length))
@@ -431,11 +507,14 @@ dime_find_record(const GSList *records, const char *type, const char *id)
 		return record;
 	}
 
-	g_message("Could not find record (type=\"%s\", id=%s%s%s)",
-		type,
-		id ? "\"" : "",
-		id ? id : "<none>",
-		id ? "\"" : "");
+	if (GNET_PROPERTY(tigertree_debug)) {
+		g_message("Could not find record (type=\"%s\", id=%s%s%s)",
+			type,
+			id ? "\"" : "",
+			id ? id : "<none>",
+			id ? "\"" : "");
+		thex_dump_dime_records(records);
+	}
 
 	return NULL;
 }
@@ -451,6 +530,9 @@ thex_download_finished(struct thex_download *ctx)
 
 	ctx->finished = TRUE;
 
+	g_assert(ctx->pos <= ctx->data_size);
+	ctx->data_size = ctx->pos;	/* Amount which is actually valid */
+
 	records = dime_parse_records(ctx->data, ctx->data_size);
 	if (records) {
 		const struct dime_record *record;
@@ -459,28 +541,55 @@ thex_download_finished(struct thex_download *ctx)
 		size_t size;
 		
 		record = dime_find_record(records, "text/xml", NULL);
-		if (NULL == record)
+		if (NULL == record) {
+			if (GNET_PROPERTY(tigertree_debug)) {
+				dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
+			}
 			goto finish;
+		}
 
 		data = dime_record_data(record);
 		size = dime_record_data_length(record);
 		hashtree_id = thex_download_handle_xml(ctx, data, size);
-		if (NULL == hashtree_id)
-			goto finish;
+		if (NULL == hashtree_id) {
+			if (GNET_PROPERTY(tigertree_debug)) {
+				g_message("Could not determine hashtree ID");
+				dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
+			}
+			/* Bug workaround:
+			 * Try without an ID. GnucDNA 1.1.1.4 sends truncated XML with
+			 * a missing closing tag.
+			 */
+		}
 
 		record = dime_find_record(records, THEX_TREE_TYPE, hashtree_id);
+		if (NULL == record && NULL != hashtree_id) {
+			/* Bug workaround:
+			 * Ignore the ID and fetch the first record with a matching
+			 * type. BearShare 5.2 prepends a bogus double-quote to the ID.
+			 */
+			record = dime_find_record(records, THEX_TREE_TYPE, NULL);
+		}
 		G_FREE_NULL(hashtree_id);
 
-		if (NULL == record)
+		if (NULL == record) {
+			if (GNET_PROPERTY(tigertree_debug)) {
+				dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
+			}
 			goto finish;
+		}
 
 		data = dime_record_data(record);
 		size = dime_record_data_length(record);
+
 		if (!thex_download_handle_hashtree(ctx, data, size))
 			goto finish;
 
 	} else {
-		g_message("Could not parse DIME records");
+		if (GNET_PROPERTY(tigertree_debug)) {
+			g_message("Could not parse DIME records");
+			dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
+		}
 		goto finish;
 	}
 	success = TRUE;
