@@ -1,5 +1,5 @@
 /*
- * $Id: tx_link.h,v 1.3 2005/06/25 01:37:41 daichik Exp $
+ * $Id: tx_link.h 12583 2006-12-06 05:42:15Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,10 +36,32 @@
 #ifndef _core_tx_link_h_
 #define _core_tx_link_h_
 
+#include "common.h"
+
 #include "tx.h"
+#include "if/core/bsched.h"
 
 const struct txdrv_ops *tx_link_get_ops(void);
 
+/**
+ * Callbacks used by the link layer.
+ */
+struct tx_link_cb {
+	void (*add_tx_written)(gpointer owner, gint amount);
+	void (*eof_remove)(gpointer owner, const gchar *reason, ...);
+	void (*eof_shutdown)(gpointer owner, const gchar *reason, ...);
+	void (*unflushq)(gpointer owner);
+};
+
+/**
+ * Arguments to be passed when the layer is intantiated.
+ */
+struct tx_link_args {
+	const struct tx_link_cb *cb;	/**< Callbacks */
+	struct wrap_io *wio;			/**< I/O wrapping routines */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
+};
+
 #endif	/* _core_tx_link_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
