@@ -1,5 +1,5 @@
 /*
- * $Id: version.c,v 1.10 2005/05/04 17:37:24 graaff Exp $
+ * $Id: version.c 15126 2008-03-10 14:35:34Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -28,13 +28,14 @@
  * @file
  *
  * Version management.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: version.c,v 1.10 2005/05/04 17:37:24 graaff Exp $");
-
-#include <sys/utsname.h>		/* For uname() */
+RCSID("$Id: version.c 15126 2008-03-10 14:35:34Z cbiere $")
 
 #include "version.h"
 #include "token.h"
@@ -43,16 +44,20 @@ RCSID("$Id: version.c,v 1.10 2005/05/04 17:37:24 graaff Exp $");
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
+#include "if/core/main.h"
+
 #include "lib/atoms.h"
 #include "lib/getdate.h"
 #include "lib/misc.h"
 #include "lib/glib-missing.h"
+#include "lib/tm.h"
+#include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 #define SECS_PER_DAY	86400
 
-gchar *version_string = NULL;
-gchar *version_short_string = NULL;
+const gchar *version_string = NULL;
+const gchar *version_short_string = NULL;
 
 static version_t our_version;
 static version_t last_rel_version;
@@ -84,13 +89,13 @@ static void
 version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
 {
 	printf("VERSION%s \"%s\":\n"
-		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u\n",
+		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u build=%u\n",
 		cmptag, str, ver->major, ver->minor, ver->patchlevel,
-		ver->tag ? ver->tag : ' ', ver->taglevel);
+		ver->tag ? ver->tag : ' ', ver->taglevel, ver->build);
 }
 
 /**
- * Return a user-friendly description of the version.
+ * @return a user-friendly description of the version.
  * NB: returns pointer to static data.
  */
 const gchar *
@@ -110,6 +115,9 @@ version_str(const version_t *ver)
 			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
 	}
 
+	if (ver->build)
+		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
+
 	if (ver->timestamp) {
 		struct tm *tmp = localtime(&ver->timestamp);
 		rw += gm_snprintf(&str[rw], sizeof(str)-rw, " (%d-%02d-%02d)",
@@ -159,7 +167,7 @@ version_stamp(const gchar *str, version_t *ver)
 			 */
 
 			g_strlcpy(stamp, p, MIN(size, sizeof(stamp)));
-			ver->timestamp = date2time(stamp, time(NULL));
+			ver->timestamp = date2time(stamp, tm_time());
 
 			if (ver->timestamp == -1) {
 				ver->timestamp = 0;
@@ -175,13 +183,14 @@ version_stamp(const gchar *str, version_t *ver)
  * Parse gtk-gnutella's version number in User-Agent/Server string `str'
  * and extract relevant information into `ver'.
  *
- * Returns TRUE if we parsed a gtk-gnutella version correctly, FALSE if we
+ * @returns TRUE if we parsed a gtk-gnutella version correctly, FALSE if we
  * were not facing a gtk-gnutella version, or if we did not recognize it.
  */
 static gboolean
 version_parse(const gchar *str, version_t *ver)
 {
 	const gchar *v;
+	gint error;
 
 	/*
 	 * Modern version numbers are formatted like this:
@@ -200,62 +209,78 @@ version_parse(const gchar *str, version_t *ver)
 	 *    gtk-gnutella/0.90.1b2 (24/06/2002; X11; Linux 2.4.18-2emi i686)
 	 *
 	 * where the third number is the "patchlevel".
+	 *
+	 * Starting 2006-08-26, the user-agent string includes the SVN revision
+	 * at the time of the build.  To be compatible with the servents out
+	 * there, the version is included as an optional tag following the date.
+	 *
+	 *    gtk-gnutella/0.85 (04/04/2002; r3404; X11; FreeBSD 4.6-STABLE i386)
+	 *
+	 * However, we also start parsing the new format that we shall use when
+	 * all current GTKG out there have expired:
+	 *
+	 *    gtk-gnutella/0.85-3404 (04/04/2002; X11; FreeBSD 4.6-STABLE i386)
+	 *    gtk-gnutella/0.90b2-3404 (04/04/2002; X11; FreeBSD 4.6-STABLE i386)
+	 *
+	 * where the '-3404' introduces the build number.
 	 */
 
-	if (NULL == (v = is_strprefix(str, "gtk-gnutella/")))
+	if (NULL == (v = is_strprefix(str, GTA_PRODUCT_NAME "/")))
 		return FALSE;
 
-	if (
-		5 == sscanf(v, "%u.%u.%u%c%u",
-			&ver->major, &ver->minor, &ver->patchlevel,
-			&ver->tag, &ver->taglevel)
-	)
-		goto ok;
+	/*
+	 * Parse "major.minor", followed by optional ".patchlevel".
+	 */
 
-	ver->taglevel = 0;
+	ver->major = parse_uint32(v, &v, 10, &error);
+	if (error)
+		return FALSE;
 
-	if (
-		4 == sscanf(v, "%u.%u.%u%c",
-			&ver->major, &ver->minor, &ver->patchlevel, &ver->tag)
-	) {
-		if (!isalpha(ver->tag))		/* Not a letter */
-			ver->tag = '\0';
-		goto ok;
+	if (*v++ != '.')
+		return FALSE;
+
+	ver->minor = parse_uint32(v, &v, 10, &error);
+	if (error)
+		return FALSE;
+
+	if ('.' == *v) {
+		v++;
+		ver->patchlevel = parse_uint32(v, &v, 10, &error);
+		if (error)
+			return FALSE;
+	} else
+		ver->patchlevel = 0;
+
+	/*
+	 * Parse optional tag letter "x" followed by optional "taglevel".
+	 */
+
+	if (is_ascii_alpha(*v)) {
+		ver->tag = *v++;
+		if (is_ascii_digit(*v)) {
+			ver->taglevel = parse_uint32(v, &v, 10, &error);
+			if (error)
+				return FALSE;
+		} else
+			ver->taglevel = 0;
+	} else {
+		ver->tag = '\0';
+		ver->taglevel = 0;
 	}
 
-	ver->tag = '\0';
+	/*
+	 * Parse optional "-build".
+	 */
 
-	if (
-		3 == sscanf(v, "%u.%u.%u",
-			&ver->major, &ver->minor, &ver->patchlevel)
-	)
-		goto ok;
+	if ('-' == *v) {
+		v++;
+		ver->build = parse_uint32(v, &v, 10, &error);
+		if (error)
+			return FALSE;
+	} else
+		ver->build = 0;
 
-	ver->patchlevel = 0;
-
-	if (
-		4 == sscanf(v, "%u.%u%c%u",
-			&ver->major, &ver->minor, &ver->tag, &ver->taglevel)
-	)
-		goto ok;
-
-	ver->taglevel = 0;
-
-	if (3 == sscanf(v, "%u.%u%c", &ver->major, &ver->minor, &ver->tag)) {
-		if (!isalpha(ver->tag))		/* Not a letter */
-			ver->tag = '\0';
-		goto ok;
-	}
-
-	ver->tag = '\0';
-
-	if (2 == sscanf(v, "%u.%u", &ver->major, &ver->minor))
-		goto ok;
-
-	return FALSE;
-
-ok:
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		version_dump(str, ver, "#");
 
 	return TRUE;
@@ -263,7 +288,7 @@ ok:
 
 /**
  * Compare two tag chars, assuming version numbers are equal.
- * Returns -1, 0 or +1 depending on the sign of "a - b".
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 static gint
 version_tagcmp(guchar a, guchar b)
@@ -277,18 +302,18 @@ version_tagcmp(guchar a, guchar b)
 	if (b == '\0')			/* Stable release has no tag */
 		return -1;
 
-	if (a == 'u')			/* Unstable from CVS */
+	if (a == 'u')			/* Unstable from SVN */
 		return -1;
 
-	if (b == 'u')			/* Unstable from CVS */
+	if (b == 'u')			/* Unstable from SVN */
 		return +1;
 
 	return a < b ? -1 : +1;	/* 'a' or 'b' for Alpha / Beta */
 }
 
 /**
- * Compare two gtk-gnutella versions, timestamp not withstanding.
- * Returns -1, 0 or +1 depending on the sign of "a - b".
+ * Compare two gtk-gnutella versions, timestamp and build not withstanding.
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
 gint
 version_cmp(const version_t *a, const version_t *b)
@@ -310,9 +335,24 @@ version_cmp(const version_t *a, const version_t *b)
 	return a->major < b->major ? -1 : +1;
 }
 
+/**
+ * Compare two gtk-gnutella versions, including build number (but not
+ * the version timestamps).  In theory, we could simply compare build numbers
+ * but we can't ensure they won't ever be reset one day.
+ *
+ * @returns -1, 0 or +1 depending on the sign of "a - b".
+ */
+gint
+version_build_cmp(const version_t *a, const version_t *b)
+{
+	gint cmp = version_cmp(a, b);
+	return 0 == cmp ? CMP(a->build, b->build) : cmp;
+}
+
 /**
  * Parse vendor string and fill supplied version structure `vs'.
- * Returns OK if we were able to parse correctly.
+ *
+ * @returns OK if we were able to parse correctly.
  */
 gboolean
 version_fill(const gchar *version, version_t *vs)
@@ -333,53 +373,41 @@ version_new_found(const gchar *text, gboolean stable)
 {
     static gchar last_stable[256] = "";
     static gchar last_dev[256] = "";
-    static gchar s[1024];
+	gchar s[1024];
 
     if (stable)
-        g_strlcpy(last_stable, text, sizeof(last_stable));
+        utf8_strlcpy(last_stable, text, sizeof last_stable);
     else
-        g_strlcpy(last_dev, text, sizeof(last_dev));
+        utf8_strlcpy(last_dev, text, sizeof last_dev);
 
-	gm_snprintf(s, sizeof s,
-		"%s - Newer version%s available: %s%s%s%s%s",
-		GTA_WEBSITE,
-		last_stable[0] && last_dev[0] ? "s" : "",
-		last_stable[0] ? "release " : "",
-		last_stable[0] ? last_stable : "",
-		last_stable[0] && last_dev[0] ? " / " : "",
-		last_dev[0] ? "from CVS " : "",
-		last_dev[0] ? last_dev : "");
+	if ('\0' != last_stable[0] && '\0' != last_dev[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer versions available: release %s / from SVN %s"),
+			last_stable, last_dev);
+	} else if ('\0' != last_stable[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer version available: release %s"),
+			last_stable);
+	} else if ('\0' != last_dev[0]) {
+		gm_snprintf(s, sizeof s,
+			_(" - Newer version available: from SVN %s"),
+			last_dev);
+	}
 
     gnet_prop_set_string(PROP_NEW_VERSION_STR, s);
 }
 
-/**
- * Check whether GTKG version is newer than supplied timestamp.
- */
-gboolean
-version_newer(const gchar *str, time_t stamp)
-{
-	version_t version;
-
-	if (!version_parse(str, &version))
-		return FALSE;			/* Not gtk-gnutella, or unparseable */
-
-	version_stamp(str, &version);
-
-	return version.timestamp >= stamp;
-}
-
 /**
  * Check version of servent, and if it's a gtk-gnutella more recent than we
  * are, record that fact and change the status bar.
  *
- * The `ip' is being passed solely for the tok_version_valid() call.
+ * The `addr' is being passed solely for the tok_version_valid() call.
  *
- * Returns TRUE if we properly checked the version, FALSE if we got something
+ * @returns TRUE if we properly checked the version, FALSE if we got something
  * looking as gtk-gnutella but which failed the token-based sanity checks.
  */
 gboolean
-version_check(const gchar *str, const gchar *token, guint32 ip)
+version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 {
 	version_t their_version;
 	version_t *target_version;
@@ -400,7 +428,7 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 
 	cmp = version_cmp(target_version, &their_version);
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		version_dump(str, &their_version,
 			cmp == 0 ? "=" :
 			cmp > 0 ? "-" : "+");
@@ -411,7 +439,7 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 
 	version_stamp(str, &their_version);
 
-	if (dbg > 6)
+	if (GNET_PROPERTY(dbg) > 6)
 		printf("VERSION time=%d\n", (gint) their_version.timestamp);
 
 	/*
@@ -423,18 +451,32 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 		tok_error_t error;
 
 		if (token == NULL) {
-            if (dbg) {
+            if (GNET_PROPERTY(dbg)) {
                 g_message("got GTKG vendor string \"%s\" without token!", str);
             }
 			return FALSE;	/* Can't be correct */
 		}
 
-		error = tok_version_valid(str, token, strlen(token), ip);
+		error = tok_version_valid(str, token, strlen(token), addr);
+
+		/*
+		 * Unfortunately, if our token has expired, we can no longer
+		 * validate the tokens of the remote peers, since they are using
+		 * a different set of keys.
+		 *
+		 * This means an expired GTKG will blindly trust well-formed remote
+		 * tokens at face value.  But it's their fault, they should not run
+		 * an expired version!
+		 *		--RAM, 2005-12-21
+		 */
+
+		if (error == TOK_BAD_KEYS && tok_is_ancient(tm_time()))
+			error = TOK_OK;		/* Our keys have expired, cannot validate */
 
 		if (error != TOK_OK) {
-            if (dbg) {
+            if (GNET_PROPERTY(dbg)) {
                 g_message("vendor string \"%s\" [%s] has wrong token "
-                    "\"%s\": %s ", str, ip_to_gchar(ip), token,
+                    "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
             }
 			return FALSE;
@@ -452,20 +494,21 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 
 	/*
 	 * If timestamp is greater and we were comparing against a stable
-	 * release, and cmp == 0, then this means an update in CVS about
+	 * release, and cmp == 0, then this means an update in SVN about
 	 * a "released" version, probably alpha/beta.
 	 */
 
 	if (
 		cmp == 0 &&
-		delta_time(their_version.timestamp, target_version->timestamp) > 0 &&
+		(delta_time(their_version.timestamp, target_version->timestamp) > 0
+			|| their_version.build > target_version->build) &&
 		target_version == &last_rel_version
 	) {
-		if (dbg > 6)
-			printf("VERSION is a CVS update of a release\n");
+		if (GNET_PROPERTY(dbg) > 6)
+			printf("VERSION is a SVN update of a release\n");
 
-		if (version_cmp(&last_dev_version, &their_version) > 0) {
-			if (dbg > 6)
+		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
+			if (GNET_PROPERTY(dbg) > 6)
 				printf("VERSION is less recent than latest dev we know\n");
 			return TRUE;
 		}
@@ -473,30 +516,30 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 	}
 
 	/*
-	 * Only compare a development version which has the same version
-	 * number with a more up to date version.
-	 * This will also avoid a 0.93.4u to be listed out of date if also a 0.94u
-	 * is available, while both are actually in development.
-	 * 		-- JA 15/04/2004
+	 * Their version is more recent, but is unstable -- only continue if
+	 * our version is also unstable.
 	 */
 
-	/*
-	 * Their version is more recent, but is unstable
-	 */
-	if (their_version.tag == 'u' && cmp < 0)
+	if (cmp < 0 && their_version.tag == 'u' && our_version.tag != 'u')
 		return TRUE;
 
-	if (delta_time(their_version.timestamp, target_version->timestamp) <= 0)
+	if (
+		delta_time(their_version.timestamp, target_version->timestamp) < 0 ||
+		their_version.build <= target_version->build
+	)
 		return TRUE;
 
-	if (delta_time(their_version.timestamp, our_version.timestamp) == 0)
+	if (
+		delta_time(their_version.timestamp, our_version.timestamp) == 0 &&
+		their_version.build <= our_version.build
+	)
 		return TRUE;
 
 	/*
 	 * We found a more recent version than the last version seen.
 	 */
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(dbg) > 4)
 		printf("more recent %s VERSION \"%s\"\n",
 			target_version == &last_dev_version ? "dev" : "rel", str);
 
@@ -511,7 +554,7 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 
 	version =  version_str(&their_version);
 
-	g_warning("more recent %s version of gtk-gnutella: %s",
+	g_message("more recent %s version of gtk-gnutella: %s",
 		target_version == &last_dev_version ? "development" : "released",
 		version);
 
@@ -523,41 +566,81 @@ version_check(const gchar *str, const gchar *token, guint32 ip)
 	return TRUE;
 }
 
+/**
+ * Generates the version string. This function does not require any
+ * initialization, thus may be called very early e.g., for showing
+ * version information if the executable was invoked with --version
+ * as argument.
+ *
+ * @return A pointer to a static buffer holding the version string.
+ */
+const gchar *
+version_build_string(void)
+{
+	static gboolean initialized;
+	static gchar buf[128];
+
+	if (!initialized) {
+		const gchar *sysname = "Unknown";
+		const gchar *machine = NULL;
+
+		initialized = TRUE;
+
+#ifdef HAS_UNAME
+		{
+			static struct utsname un;	/* Must survive this scope */
+
+			if (-1 != uname(&un)) {
+				sysname = un.sysname;
+				machine = un.machine;
+			} else {
+				g_warning("uname() failed: %s", g_strerror(errno));
+			}
+		}
+#endif /* HAS_UNAME */
+
+		gm_snprintf(buf, sizeof buf,
+			GTA_PRODUCT_NAME "/%s-%u (%s; %s; %s%s%s)",
+			GTA_VERSION_NUMBER, main_get_build(),
+			GTA_RELEASE, gtk_gnutella_interface(),
+			sysname,
+			machine && machine[0] ? " " : "",
+			machine ? machine : "");
+	}
+	return buf;
+}
+
 /**
  * Initialize version string.
  */
 void
 version_init(void)
 {
-	struct utsname un;
-	gchar buf[128];
-	gboolean ok;
-	time_t now = time(NULL);
+	time_t now;
 
-	if (uname(&un)) {
-		memset(&un, 0, sizeof un);
-		g_warning("uname() failed: %s:", g_strerror(errno));
+	version_string = atom_str_get(version_build_string());
+	now = tm_time();
+
+	{
+		gboolean ok;
+		ok = version_parse(version_string, &our_version);
+		g_assert(ok);
 	}
 
-	gm_snprintf(buf, sizeof(buf),
-		"gtk-gnutella/%s (%s; %s; %s %s)",
-		GTA_VERSION_NUMBER, GTA_RELEASE, GTA_INTERFACE,
-		un.sysname, un.machine);
-
-	version_string = atom_str_get(buf);
-	ok = version_parse(version_string, &our_version);
-	g_assert(ok);
-
 	g_message("%s", version_string);
 
 	version_stamp(version_string, &our_version);
 	g_assert(our_version.timestamp != 0);
 
-	gm_snprintf(buf, sizeof(buf),
-		"gtk-gnutella/%s (%s)",
-		GTA_VERSION_NUMBER, GTA_RELEASE);
+	{
+		gchar buf[128];
 
-	version_short_string = atom_str_get(buf);
+		gm_snprintf(buf, sizeof(buf),
+			GTA_PRODUCT_NAME "/%s-%u (%s)",
+			GTA_VERSION_NUMBER, main_get_build(), GTA_RELEASE);
+
+		version_short_string = atom_str_get(buf);
+	}
 
 	last_rel_version = our_version;		/* struct copy */
 	last_dev_version = our_version;		/* struct copy */
@@ -582,41 +665,10 @@ version_init(void)
 
 	if (
 		tok_is_ancient(now) ||
-		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN ||
-		(our_version.tag &&
-			delta_time(now, our_version.timestamp) > VERSION_UNSTABLE_WARN)
-	)
-		*(guint32 *) &ancient_version = TRUE;
-}
-
-/**
- * Called when the version has expired since the indicated amount.
- * If that amount is greater than our grace period, refuse to run unless
- * they set the "ancient_version_force" property explicitly.
- */
-static void
-version_maybe_refuse(gint overtime)
-{
-	gchar *force;
-	property_t prop = PROP_ANCIENT_VERSION_FORCE;
-
-	if (overtime < VERSION_ANCIENT_GRACE)
-		return;
-
-	force = gnet_prop_get_string(prop, NULL, 0);
-
-	if (0 == strcmp(force, version_string)) {
-		G_FREE_NULL(force);
-		return;
+		delta_time(now, our_version.timestamp) > VERSION_ANCIENT_WARN
+	) {
+		*deconstify_gboolean(&GNET_PROPERTY(ancient_version)) = TRUE;
 	}
-
-	/*
-	 * Sorry, they must explicitly allow us to run, we're too ancient.
-	 */
-
-	settings_ask_for_property(gnet_prop_name(prop), version_string);
-
-	/* NOTREACHED */
 }
 
 /**
@@ -631,8 +683,8 @@ version_maybe_refuse(gint overtime)
 void
 version_ancient_warn(void)
 {
-	time_t now = time(NULL);
-	gint lifetime, remain, elapsed;
+	time_t now = tm_time();
+	time_delta_t lifetime, remain, elapsed;
 	time_t s;
 
 	g_assert(our_version.timestamp != 0);	/* version_init() called */
@@ -649,26 +701,18 @@ version_ancient_warn(void)
 	elapsed = delta_time(now, our_version.timestamp);
 
 	if (elapsed > VERSION_ANCIENT_WARN || tok_is_ancient(now)) {
-		version_maybe_refuse(elapsed - VERSION_ANCIENT_WARN);
 		g_warning("version of gtk-gnutella is too old, you should upgrade!");
         gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
 		return;
 	}
 
-	if (our_version.tag && elapsed > VERSION_UNSTABLE_WARN) {
-		version_maybe_refuse(elapsed - VERSION_UNSTABLE_WARN);
-		g_warning("unstable version of gtk-gnutella is aging, please upgrade!");
-        gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
-		return;
-	}
-
 	/*
 	 * Check whether we're nearing ancient version status, to warn them
 	 * beforehand that the version will become old soon.
 	 */
 
-	lifetime = our_version.tag ? VERSION_UNSTABLE_WARN : VERSION_ANCIENT_WARN;
-	remain = lifetime - elapsed;
+	lifetime = VERSION_ANCIENT_WARN;
+	remain = delta_time(lifetime, elapsed);
 
 	g_assert(remain >= 0);		/* None of the checks above have fired */
 
@@ -682,45 +726,22 @@ version_ancient_warn(void)
 	 * that do not introduce significant Gnutella features.
 	 */
 
-	for (s = now + VERSION_ANCIENT_REMIND; s > now; s -= SECS_PER_DAY) {
+	s = time_advance(now, VERSION_ANCIENT_REMIND);
+	for (/* NOTHING */; delta_time(s, now) > 0; s -= SECS_PER_DAY) {
 		if (!tok_is_ancient(s))
 			break;
 	}
 
-	remain = MIN(remain, s - now);
+	remain = MIN(remain, delta_time(s, now));
 
 	/*
 	 * Let them know when version will expire soon...
 	 */
 
-	if (remain < VERSION_ANCIENT_REMIND)
+	if (remain < VERSION_ANCIENT_REMIND) {
         gnet_prop_set_guint32_val(PROP_ANCIENT_VERSION_LEFT_DAYS,
 			remain / SECS_PER_DAY);
-}
-
-/**
- * Check the timestamp in the GTKG version string and returns TRUE if it
- * is too old or could not be parsed, FALSE if OK.
- */
-gboolean
-version_is_too_old(const gchar *vendor)
-{
-	version_t ver;
-	time_t now = time(NULL);
-	gint d;
-
-	version_stamp(vendor, &ver);		/* Fills ver->timestamp */
-	d = delta_time(now, ver.timestamp);
-	if (d > VERSION_ANCIENT_BAN)
-		return TRUE;
-
-	if (!version_parse(vendor, &ver))	/* Fills ver->tag */
-		return TRUE;					/* Unable to parse */
-
-	if (ver.tag && d > VERSION_UNSTABLE_BAN)
-		return TRUE;
-
-	return FALSE;
+	}
 }
 
 /**
@@ -742,4 +763,4 @@ version_close(void)
 			version_str(&last_dev_version));
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
