@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.h,v 1.8 2005/11/13 10:36:12 cbiere Exp $
+ * $Id: vendors.h 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -36,7 +36,7 @@
 #ifndef _vendors_h_
 #define _vendors_h_
 
-#include <glib.h>
+#include "common.h"
 
 /***
  *** Known gnutella vendor codes
@@ -74,6 +74,7 @@
 #define T_GNUC	MAKE_CODE('G','N','U','C')
 #define T_GNUM  MAKE_CODE('G','N','U','M')
 #define T_GNUT	MAKE_CODE('G','N','U','T')
+#define T_GNZL	MAKE_CODE('G','N','Z','L')
 #define T_GTKG	MAKE_CODE('G','T','K','G')
 #define T_HSLG	MAKE_CODE('H','S','L','G')
 #define T_HUIT  MAKE_CODE('H','U','I','T')
@@ -104,6 +105,7 @@
 #define T_QTEL	MAKE_CODE('Q','T','E','L')
 #define T_RASP  MAKE_CODE('R','A','S','P')
 #define T_RAZA	MAKE_CODE('R','A','Z','A')
+#define T_RAZB	MAKE_CODE('R','A','Z','B')
 #define T_SHNB	MAKE_CODE('S','H','N','B')
 #define T_SNOW	MAKE_CODE('S','N','O','W')
 #define T_SNUT	MAKE_CODE('S','N','U','T')
@@ -130,14 +132,13 @@
  */
 #define VENDOR_CODE_CMP(a, b) CMP(a, b)
 
-union vendor_code {
+typedef struct vendor_code {
 	guint32 be32;	/**< Always big-endian order; for convenient '=' and '!=' */
-	guchar	b[4];	/**< Holds a four-Letter Gnutella vendor ID */
-};
+} vendor_code_t;
 
 const gchar *vendor_code_str(guint32 code);
-const gchar *lookup_vendor_name(union vendor_code code);
-gboolean is_vendor_known(union vendor_code code);
+const char *vendor_get_name(guint32 code);
+gboolean is_vendor_known(vendor_code_t code);
 
 void vendor_init(void);
 
