@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h,v 1.3 2005/06/29 14:24:27 daichik Exp $
+ * $Id: downloads.h 14372 2007-08-08 21:43:46Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -26,31 +26,8 @@
 #ifndef _gtk_downloads_h_
 #define _gtk_downloads_h_
 
-#include "gui.h"
 #include "downloads_common.h"
 
-struct download;
-
-/** Indicates that a dl node is a header node */
-#define DL_GUI_IS_HEADER GINT_TO_POINTER(0x1)
-
-/*
- * Global Functions
- */
-
-void downloads_gui_init(void);
-void downloads_gui_shutdown(void);
-
-#ifdef USE_GTK1
-GList *downloads_gui_collect_ctree_data(GtkCTree *ctree, GList *node_list,
-	gboolean unselect, gboolean add_children);
-void downloads_gui_expand_all(GtkCTree *ctree);
-void downloads_gui_collapse_all(GtkCTree *ctree);
-#else
-void downloads_gui_expand_all(GtkTreeView *tree_view);
-void downloads_gui_collapse_all(GtkTreeView *tree_view);
-#endif
-
 #endif /* _gtk_downloads_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
