@@ -1,5 +1,5 @@
 /*
- * $Id: visual_progress.c,v 1.37 2005/11/03 08:51:51 rmanfredi Exp $
+ * $Id: visual_progress.c 14173 2007-07-24 03:34:25Z cbiere $
  *
  * Copyright (c) 2003-2005, Hans de Graaff
  *
@@ -44,7 +44,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: visual_progress.c,v 1.37 2005/11/03 08:51:51 rmanfredi Exp $");
+RCSID("$Id: visual_progress.c 14173 2007-07-24 03:34:25Z cbiere $")
 
 #include "visual_progress.h"
 
@@ -52,6 +52,7 @@ RCSID("$Id: visual_progress.c,v 1.37 2005/11/03 08:51:51 rmanfredi Exp $");
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/atoms.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
@@ -74,29 +75,31 @@ typedef struct vp_context {
  */
 typedef struct vp_info {
     gnet_fi_t fi_handle;
-    gchar *file_name;
-    filesize_t file_size;
+    const gchar *filename;		/* atom */
     GSList *chunks_list;
 	GSList *chunks_initial;
 	GSList *ranges_list;
-	filesize_t done_initial;
     vp_context_t *context;
+    filesize_t filesize;
+	filesize_t done_initial;
 } vp_info_t;
 
 static GHashTable *vp_info_hash; /**< Hash table with our cached fileinfo info */
 
-static GdkColor done;       /**< Pre-filled color (green) for DONE chunks */
-static GdkColor done_old;	/**< Pre-filled color (dull green) for DONE
+static struct {
+	GdkColor base;      /**< Theme-defined background color */
+	GdkColor done;       /**< Pre-filled color (green) for DONE chunks */
+	GdkColor done_old;	/**< Pre-filled color (dull green) for DONE
 								 chunks from previous sessions */
-static GdkColor busy;       /**< Pre-filled color (yellow) for BUSY chunks */
-static GdkColor arrow;      /**< Pre-filled color (blue) for start of BUSY */
-static GdkColor empty;      /**< Pre-filled color (red) for EMPTY chunks */
-static GdkColor black;      /**< Pre-filled color (black) for general drawing */
-static GdkColor available;  /**< Pre-filled color (blue) available on network */
-static GdkColor nosize;     /**< Pre-filled color (gray) indicates
+	GdkColor busy;       /**< Pre-filled color (yellow) for BUSY chunks */
+	GdkColor arrow;      /**< Pre-filled color (blue) for start of BUSY */
+	GdkColor empty;      /**< Pre-filled color (red) for EMPTY chunks */
+	GdkColor black;      /**< Pre-filled color (black) for general drawing */
+	GdkColor available;  /**< Pre-filled color (blue) available on network */
+	GdkColor nosize;     /**< Pre-filled color (gray) indicates
 								 chunk information is not available
 								 (e.g. file size == 0 */
-static GdkColor *base;      /**< Theme-defined background color */
+} colors;
 
 /**
  * The visual progress context for drawing fileinfo information.
@@ -124,16 +127,16 @@ vp_draw_rectangle(vp_info_t *v, filesize_t from, filesize_t to,
 	 * neither should be zero when we end up here, so this can be
 	 * considered a bug somewhere in the calling code.
 	 *
-	 * file_size should be set in the fileinfo code. For files with
-	 * unknown size the file_size == 0, but in this case
+	 * filesize should be set in the fileinfo code. For files with
+	 * unknown size the filesize == 0, but in this case
 	 * vp_draw_fi_progress catches this case.
 	 */
-	g_assert(v->file_size);
+	g_assert(v->filesize > 0);
 
-	s_from = (gfloat) from * v->context->widget->allocation.width
-		/ v->file_size;
-	s_to   = (gfloat) to   * v->context->widget->allocation.width
-		/ v->file_size;
+	s_from = (1.0 * from * v->context->widget->allocation.width)
+		/ v->filesize;
+	s_to   = (1.0 * to   * v->context->widget->allocation.width)
+		/ v->filesize;
 
     gdk_draw_rectangle(v->context->drawable, v->context->gc, TRUE,
         s_from, top, s_to - s_from, bottom);
@@ -149,14 +152,14 @@ vp_draw_chunk(gpointer data, gpointer user_data)
     vp_info_t *v = user_data;
 
     if (DL_CHUNK_EMPTY == chunk->status)
-        gdk_gc_set_foreground(v->context->gc, &empty);
+        gdk_gc_set_foreground(v->context->gc, &colors.empty);
     if (DL_CHUNK_BUSY == chunk->status)
-        gdk_gc_set_foreground(v->context->gc, &busy);
+        gdk_gc_set_foreground(v->context->gc, &colors.busy);
     if (DL_CHUNK_DONE == chunk->status) {
         if (chunk->old)
-            gdk_gc_set_foreground(v->context->gc, &done_old);
+            gdk_gc_set_foreground(v->context->gc, &colors.done_old);
         else
-            gdk_gc_set_foreground(v->context->gc, &done);
+            gdk_gc_set_foreground(v->context->gc, &colors.done);
     }
 
     vp_draw_rectangle(v,
@@ -177,9 +180,9 @@ vp_draw_arrow(vp_info_t *v, filesize_t at)
     g_assert(v->context);
     g_assert(v->context->drawable);
 
-	g_assert(v->file_size);
+	g_assert(v->filesize);
 
-	s_at = (gfloat) at * v->context->widget->allocation.width / v->file_size;
+	s_at = (1.0 * at * v->context->widget->allocation.width) / v->filesize;
 
 	/* Fill the inside of the arrow */
 	points[0].x = s_at - VP_ARROW_HEIGHT;
@@ -188,12 +191,12 @@ vp_draw_arrow(vp_info_t *v, filesize_t at)
 	points[1].y = VP_ARROW_HEIGHT;
 	points[2].x = s_at + VP_ARROW_HEIGHT;
 	points[2].y = 0;
-	gdk_gc_set_foreground(v->context->gc, &arrow);
+	gdk_gc_set_foreground(v->context->gc, &colors.arrow);
 	gdk_draw_polygon(v->context->drawable, v->context->gc,
 		TRUE, points, G_N_ELEMENTS(points));
 
 	/* Draw a black border around the arrow */
-	gdk_gc_set_foreground(v->context->gc, &black);
+	gdk_gc_set_foreground(v->context->gc, &colors.black);
 	gdk_draw_polygon(v->context->drawable, v->context->gc,
 		FALSE, points, G_N_ELEMENTS(points));
 }
@@ -225,7 +228,7 @@ vp_draw_range (gpointer data, gpointer user_data)
 	http_range_t *range = data;
 	vp_info_t *v = user_data;
 
-	gdk_gc_set_foreground(v->context->gc, &available);
+	gdk_gc_set_foreground(v->context->gc, &colors.available);
 	vp_draw_rectangle(v,
 		range->start, range->end,
         v->context->widget->allocation.height - 3,
@@ -257,25 +260,25 @@ vp_draw_fi_progress(gboolean valid, gnet_fi_t fih)
 
 			found = g_hash_table_lookup_extended(vp_info_hash,
 				GUINT_TO_POINTER(fih), NULL, &value);
-			g_assert(found);
+    		g_return_if_fail(found);
 			g_assert(value);
 
 			v = value;
 			v->context = &fi_context;
 
-			if (v->file_size > 0) {
+			if (v->filesize > 0) {
 				g_slist_foreach(v->chunks_list, vp_draw_chunk, v);
 				g_slist_foreach(v->chunks_list, vp_draw_arrows, v);
 				g_slist_foreach(v->ranges_list, vp_draw_range, v);
 			} else {
-				gdk_gc_set_foreground(fi_context.gc, &nosize);
+				gdk_gc_set_foreground(fi_context.gc, &colors.nosize);
 				gdk_draw_rectangle(fi_context.drawable, fi_context.gc, TRUE,
 								   0, 0,
 								   fi_context.widget->allocation.width,
 								   fi_context.widget->allocation.height);
 			}
 		} else {
-			gdk_gc_set_foreground(fi_context.gc, base);
+			gdk_gc_set_foreground(fi_context.gc, &colors.base);
 			gdk_draw_rectangle(fi_context.drawable, fi_context.gc, TRUE,
 							   0, 0,
 							   fi_context.widget->allocation.width,
@@ -300,7 +303,7 @@ on_drawingarea_fi_progress_realize(GtkWidget *widget, gpointer user_data)
     g_assert(fi_context.gc);
 
 	style = gtk_widget_get_style(widget);
-	base = gdk_color_copy(&(style->base[GTK_STATE_INSENSITIVE]));
+	colors.base = style->base[GTK_STATE_INSENSITIVE];
 }
 
 /**
@@ -328,18 +331,26 @@ on_drawingarea_fi_progress_expose_event(
 static GSList *
 vp_get_chunks_initial(gnet_fi_t fih) {
 
-	GSList *result;
-	GSList *l;
+	GSList *result, *sl, *prev = NULL;
 
 	result = guc_fi_get_chunks(fih);
 
-	for (l = result; l; ) {
-		gnet_fi_chunks_t *chunk = (gnet_fi_chunks_t *) l->data;
-		l = g_slist_next(l);
+	for (sl = result; sl; /* NOTHING */) {
+		gnet_fi_chunks_t *chunk = sl->data;
+		GSList *next;
+
+		next = g_slist_next(sl);
 		if (DL_CHUNK_DONE != chunk->status) {
-			result = g_slist_remove(result, chunk);
-			wfree(chunk, sizeof(gnet_fi_chunks_t));
+			if (prev) {
+				prev = g_slist_delete_link(prev, sl);
+			} else {
+				result = g_slist_delete_link(result, sl);
+			}
+			wfree(chunk, sizeof *chunk);
+		} else {
+			prev = sl;
 		}
+		sl = next;
 	}
 
 	return result;
@@ -354,17 +365,17 @@ vp_get_chunks_initial(gnet_fi_t fih) {
 static void
 vp_gui_fi_added(gnet_fi_t fih)
 {
-    gnet_fi_info_t *fi = NULL;
-    vp_info_t *new_vp_info = NULL;
+    gnet_fi_info_t *fi;
+    vp_info_t *new_vp_info;
     gnet_fi_status_t s;
 
     fi = guc_fi_get_info(fih);
     guc_fi_get_status(fih, &s);
 
-    new_vp_info = walloc0(sizeof(*new_vp_info));
+    new_vp_info = walloc0(sizeof *new_vp_info);
     new_vp_info->fi_handle = fih;
-    new_vp_info->file_name = g_strdup(fi->file_name);
-    new_vp_info->file_size = s.size;
+    new_vp_info->filename = atom_str_get(fi->filename);
+    new_vp_info->filesize = s.size;
     new_vp_info->chunks_list = guc_fi_get_chunks(fih);
 	new_vp_info->ranges_list = guc_fi_get_ranges(fih);
 
@@ -382,6 +393,23 @@ vp_gui_fi_added(gnet_fi_t fih)
     guc_fi_free_info(fi);
 }
 
+static void
+vp_info_free(vp_info_t **v_ptr)
+{
+	g_assert(v_ptr);
+
+	if (*v_ptr) {
+		vp_info_t *v = *v_ptr;
+		
+		guc_fi_free_chunks(v->chunks_list);
+		guc_fi_free_chunks(v->chunks_initial);
+		guc_fi_free_ranges(v->ranges_list);
+		atom_str_free_null(&v->filename);
+		wfree(v, sizeof *v);
+		*v_ptr = NULL;
+	}
+}
+
 /**
  * Handle the event that a fileinfo entry has been removed.
  *
@@ -396,18 +424,12 @@ vp_gui_fi_removed(gnet_fi_t fih)
 
     found = g_hash_table_lookup_extended(vp_info_hash,
 		GUINT_TO_POINTER(fih), NULL, &value);
-    g_assert(found);
+	g_return_if_fail(found);
     g_assert(value);
+
 	v = value;
-
     g_hash_table_remove(vp_info_hash, GUINT_TO_POINTER(fih));
-
-    guc_fi_free_chunks(v->chunks_list);
-	guc_fi_free_chunks(v->chunks_initial);
-	guc_fi_free_ranges(v->ranges_list);
-	G_FREE_NULL(v->file_name);
-
-    wfree(v, sizeof(vp_info_t));
+	vp_info_free(&v);
 
     /* Forget the fileinfo handle for which we displayed progress info */
     fi_context.fih_valid = FALSE;
@@ -417,7 +439,7 @@ vp_gui_fi_removed(gnet_fi_t fih)
  * For debugging: print chunk.
  */
 static void
-vp_print_chunk(FILE *file, gnet_fi_chunks_t *c, gboolean show_old)
+vp_print_chunk(FILE *file, const gnet_fi_chunks_t *c, gboolean show_old)
 {
 	if (show_old)
 		fprintf(file, "%10s - %10s %d [%s]\n",
@@ -433,14 +455,14 @@ vp_print_chunk(FILE *file, gnet_fi_chunks_t *c, gboolean show_old)
  * For debugging: print chunk list.
  */
 static void
-vp_print_chunk_list(FILE *file, GSList *list, gchar *title)
+vp_print_chunk_list(FILE *file, const GSList *list, const gchar *title)
 {
-	GSList *l;
+	const GSList *sl;
 
 	fprintf(file, "Chunk list \"%s\":\n", title);
 
-	for (l = list; l; l = g_slist_next(l)) {
-		gnet_fi_chunks_t *c = l->data;
+	for (sl = list; sl; sl = g_slist_next(sl)) {
+		const gnet_fi_chunks_t *c = sl->data;
 		vp_print_chunk(file, c, FALSE);
 	}
 
@@ -461,7 +483,7 @@ vp_create_chunk(filesize_t from, filesize_t to,
 {
 	gnet_fi_chunks_t *chunk;
 
-	chunk = walloc(sizeof(gnet_fi_chunks_t));
+	chunk = walloc(sizeof *chunk);
 	chunk->from = from;
 	chunk->to = to;
 	chunk->status = status;
@@ -479,19 +501,18 @@ vp_create_chunk(filesize_t from, filesize_t to,
  * Assert that a chunks list confirms to the assumptions.
  */
 static gboolean
-vp_assert_chunks_list(GSList *list, gnet_fi_info_t *fi)
+vp_assert_chunks_list(const GSList *list, const gnet_fi_info_t *fi)
 {
-	GSList *l;
-	gnet_fi_chunks_t *chunk;
+	const GSList *sl;
 	filesize_t last = 0;
 
-	for (l = list; l; l = g_slist_next(l)) {
-		chunk = (gnet_fi_chunks_t *) l->data;
+	for (sl = list; sl; sl = g_slist_next(sl)) {
+		const gnet_fi_chunks_t *chunk = sl->data;
+
 		if (last != chunk->from) {
-			g_warning("BAD CHUNK LIST for \"%s\"", fi->file_name);
+			g_warning("BAD CHUNK LIST for \"%s\"", fi->filename);
 			vp_print_chunk_list(stderr, list, "Chunks list");
 			return FALSE;
-			break;
 		}
 		last = chunk->to;
 	}
@@ -521,7 +542,7 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 
     found = g_hash_table_lookup_extended(vp_info_hash,
 		GUINT_TO_POINTER(fih), NULL, &value);
-    g_assert(found);
+	g_return_if_fail(found);
     g_assert(value);
 	v = value;
 
@@ -563,8 +584,8 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 
 	while (old || new) {
 		if (old && new) {
-			oc = (gnet_fi_chunks_t *) old->data;
-			nc = (gnet_fi_chunks_t *) new->data;
+			oc = old->data;
+			nc = new->data;
 
 			/*
 			 * Skip over chunks below the highest mark, they are no longer
@@ -677,14 +698,14 @@ vp_gui_fi_status_changed(gnet_fi_t fih)
 			 * the chunks list.
 			 */
 			if (old) {
-				oc = (gnet_fi_chunks_t *) old->data;
+				oc = old->data;
 				v->chunks_list = g_slist_append(v->chunks_list,
 					vp_create_chunk(oc->from, oc->to, oc->status, TRUE));
 				old = g_slist_next(old);
 			}
 
 			if (new) {
-				nc = (gnet_fi_chunks_t *) new->data;
+				nc = new->data;
 				v->chunks_list = g_slist_append(v->chunks_list,
 					vp_create_chunk(nc->from, nc->to, nc->status, FALSE));
 				new = g_slist_next(new);
@@ -719,7 +740,7 @@ vp_gui_fi_ranges_changed(gnet_fi_t fih)
 
     found = g_hash_table_lookup_extended(vp_info_hash,
 		GUINT_TO_POINTER(fih), NULL, &value);
-    g_assert(found);
+    g_return_if_fail(found);
     g_assert(value);
 	v = value;
 
@@ -736,15 +757,14 @@ vp_gui_fi_ranges_changed(gnet_fi_t fih)
  * Free the vp_info_t structs in the vp_info_hash.
  */
 void
-vp_free_key_value (gpointer key, gpointer value, gpointer user_data)
+vp_free_key_value(gpointer key, gpointer value, gpointer user_data)
 {
+	vp_info_t *v = value;
+
 	(void) key;
 	(void) user_data;
-    guc_fi_free_chunks(((vp_info_t *) value)->chunks_list);
-	guc_fi_free_chunks(((vp_info_t *) value)->chunks_initial);
-	guc_fi_free_ranges(((vp_info_t *) value)->ranges_list);
-	G_FREE_NULL(((vp_info_t *) value)->file_name);
-    wfree(value, sizeof(vp_info_t));
+
+	vp_info_free(&v);
 }
 
 /**
@@ -774,22 +794,22 @@ vp_gui_init(void)
 
 	cmap = gdk_colormap_get_system();
     g_assert(cmap);
-    gdk_color_parse("green4", &done_old);
-    gdk_colormap_alloc_color(cmap, &done_old, FALSE, TRUE);
-    gdk_color_parse("green", &done);
-    gdk_colormap_alloc_color(cmap, &done, FALSE, TRUE);
-    gdk_color_parse("yellow2", &busy);
-    gdk_colormap_alloc_color(cmap, &busy, FALSE, TRUE);
-	gdk_color_parse("light sky blue", &arrow);
-	gdk_colormap_alloc_color(cmap, &arrow, FALSE, TRUE);
-    gdk_color_parse("red2", &empty);
-    gdk_colormap_alloc_color(cmap, &empty, FALSE, TRUE);
-    gdk_color_parse("black", &black);
-    gdk_colormap_alloc_color(cmap, &black, FALSE, TRUE);
-	gdk_color_parse("blue", &available);
-	gdk_colormap_alloc_color(cmap, &available, FALSE, TRUE);
-	gdk_color_parse("gray", &nosize);
-	gdk_colormap_alloc_color(cmap, &nosize, FALSE, TRUE);
+    gdk_color_parse("green4", &colors.done_old);
+    gdk_colormap_alloc_color(cmap, &colors.done_old, FALSE, TRUE);
+    gdk_color_parse("green", &colors.done);
+    gdk_colormap_alloc_color(cmap, &colors.done, FALSE, TRUE);
+    gdk_color_parse("yellow2", &colors.busy);
+    gdk_colormap_alloc_color(cmap, &colors.busy, FALSE, TRUE);
+	gdk_color_parse("light sky blue", &colors.arrow);
+	gdk_colormap_alloc_color(cmap, &colors.arrow, FALSE, TRUE);
+    gdk_color_parse("red2", &colors.empty);
+    gdk_colormap_alloc_color(cmap, &colors.empty, FALSE, TRUE);
+    gdk_color_parse("black", &colors.black);
+    gdk_colormap_alloc_color(cmap, &colors.black, FALSE, TRUE);
+	gdk_color_parse("blue", &colors.available);
+	gdk_colormap_alloc_color(cmap, &colors.available, FALSE, TRUE);
+	gdk_color_parse("gray", &colors.nosize);
+	gdk_colormap_alloc_color(cmap, &colors.nosize, FALSE, TRUE);
 
     /*
      * No progress fih has been seen yet
@@ -805,19 +825,19 @@ vp_gui_shutdown(void)
 {
     guc_fi_remove_listener(vp_gui_fi_removed, EV_FI_REMOVED);
     guc_fi_remove_listener(vp_gui_fi_added, EV_FI_ADDED);
-    guc_fi_remove_listener(vp_gui_fi_status_changed,
-		EV_FI_STATUS_CHANGED);
+    guc_fi_remove_listener(vp_gui_fi_status_changed, EV_FI_STATUS_CHANGED);
     guc_fi_remove_listener(vp_gui_fi_status_changed,
 		EV_FI_STATUS_CHANGED_TRANSIENT);
     guc_fi_remove_listener(vp_gui_fi_ranges_changed, EV_FI_RANGES_CHANGED);
 
     g_hash_table_foreach(vp_info_hash, vp_free_key_value, NULL);
     g_hash_table_destroy(vp_info_hash);
+	vp_info_hash = NULL;
 }
 
 /*
  * Local Variables:
  * tab-width:4
  * End:
- * vi: set ts=4:
+ * vi: set ts=4 sw=4 cindent:
  */
